@@ -46,6 +46,7 @@ use Time::HiRes qw( usleep );
 use URI;
 
 our $uri;
+my $use_optics;
 
 sub open {
   my $self = shift;
@@ -130,6 +131,8 @@ sub open {
     } else {
       Debug('No headers line');
     } # end if headers
+    # For parent get
+    $$self{BaseURL} = $uri->canonical();
   } else {
     Debug('Failed to open '.$uri->canonical().$url.' status: '.$res->status_line());
   } # end if $res->status_line() eq '401 Unauthorized'
@@ -139,19 +142,7 @@ sub sendCmd {
   my $self = shift;
   my $cmd = shift;
 
-  $self->printMsg($cmd, 'Tx');
-
-  my $url = $uri->canonical().$cmd;
-  my $res = $self->{ua}->get($url);
-
-  if ( $res->is_success ) {
-    Debug('sndCmd command: '.$url.' content: '.$res->content);
-    return !undef;
-  }
-
-  Error("Error cmd $url failed: '".$res->status_line()."'");
-
-  return undef;
+  return $self->get($cmd);
 }
 
 sub cameraReset {
@@ -390,17 +381,46 @@ sub focusRelNear {
   my $self = shift;
   my $params = shift;
   my $step = $self->getParam($params, 'step');
-  Debug('Focus Near');
-  my $cmd = "/axis-cgi/com/ptz.cgi?rfocus=-$step";
-  $self->sendCmd($cmd);
+  Debug('Focus Rel Near');
+   if ($use_optics) {
+    my $cmd = "/axis-cgi/opticssetup.cgi?rfocus=-minstep";
+    $self->sendCmd($cmd);
+  } else {
+    my $cmd = "/axis-cgi/com/ptz.cgi?rfocus=-$step";
+    my $res = $self->sendCmd($cmd);
+    if ($res->content ne 'ok') {
+      $use_optics = 1;
+      $self->focusRelNear($self, $params);
+    }
+  }
 }
 
 sub focusRelFar {
   my $self = shift;
   my $params = shift;
   my $step = $self->getParam($params, 'step');
-  Debug('Focus Far');
-  my $cmd = "/axis-cgi/com/ptz.cgi?rfocus=$step";
+  Debug('FocusRelFar');
+  if ($use_optics) {
+    my $cmd = "/axis-cgi/opticssetup.cgi?rfocus=minstep";
+    $self->sendCmd($cmd);
+  } else {
+    my $cmd = "/axis-cgi/com/ptz.cgi?rfocus=$step";
+    my $res = $self->sendCmd($cmd);
+    if ($res->content ne 'ok') {
+      $use_optics = 1;
+      $self->focusRelNear($self, $params);
+    }
+  }
+}
+
+sub focusAbs {
+  # Takes a value 0-1 in small decimal incremements
+  my $self = shift;
+  my $params = shift;
+  my $step = $self->getParam($params, 'step');
+  # step comes in as an integer, needs to be 0-1
+  Debug('Focus Abs');
+  my $cmd = "/axis-cgi/opticssetup.cgi.cgi?afocus=$step";
   $self->sendCmd($cmd);
 }
 
@@ -484,6 +504,12 @@ sub presetHome {
   $self->sendCmd($cmd);
 }
 
+sub reboot {
+  my $self = shift;
+  my $response = $self->sendCmd('/axis-cgi/restart.cgi');
+  return $response->is_success;
+}
+
 1;
 __END__
 # Below is stub documentation for your module. You'd better edit it!
