@@ -33,6 +33,8 @@ require ZoneMinder::Object;
 require ZoneMinder::Storage;
 require ZoneMinder::Frame;
 require ZoneMinder::Monitor;
+require ZoneMinder::Event_Tag;
+require ZoneMinder::Tag;
 require Date::Manip;
 require File::Find;
 require File::Path;
@@ -43,6 +45,7 @@ require Date::Parse;
 require POSIX;
 use Date::Format qw(time2str);
 use Time::HiRes qw(gettimeofday tv_interval stat);
+use Scalar::Util qw(looks_like_number);
 
 #our @ISA = qw(ZoneMinder::Object);
 use parent qw(ZoneMinder::Object);
@@ -263,7 +266,7 @@ sub createIdFile {
 }
 
 sub GenerateVideo {
-  my ( $self, $rate, $fps, $scale, $size, $overwrite, $format ) = @_;
+  my ( $self, $rate, $fps, $scale, $size, $overwrite, $format, $transforms ) = @_;
 
   my $event_path = $self->Path();
   chdir($event_path);
@@ -276,14 +279,14 @@ sub GenerateVideo {
     $file_rate =~ s/_00//;
     $file_rate =~ s/(_\d+)0+$/$1/;
     $file_rate = 'r'.$file_rate;
-    push( @file_parts, $file_rate );
+    push @file_parts, $file_rate;
   } elsif ( $fps ) {
     my $file_fps = $fps;
     $file_fps =~ s/\./_/;
     $file_fps =~ s/_00//;
     $file_fps =~ s/(_\d+)0+$/$1/;
     $file_fps = 'R'.$file_fps;
-    push( @file_parts, $file_fps );
+    push @file_parts, $file_fps;
   }
 
   if ( $scale ) {
@@ -297,7 +300,11 @@ sub GenerateVideo {
     my $file_size = 'S'.$size;
     push @file_parts, $file_size;
   }
-  my $video_file = join('-', $video_name, $file_parts[0], $file_parts[1] ).'.'.$format;
+  my @transforms = split(',', $transforms);
+  foreach (@transforms) {
+    push @file_parts, $_;
+  }
+  my $video_file = join('-', $video_name, @file_parts).'.'.$format;
   if ( $overwrite || !-s $video_file ) {
     Info("Creating video file $video_file for event $self->{Id}");
 
@@ -328,7 +335,10 @@ sub GenerateVideo {
       .$Config{ZM_FFMPEG_INPUT_OPTIONS}
     .' -i ' . ( $$self{DefaultVideo} ? $$self{DefaultVideo} : '%0'.$Config{ZM_EVENT_IMAGE_DIGITS} .'d-capture.jpg' )
 #. " -f concat -i /tmp/event_files.txt"
-      ." -s $video_size "
+    #
+   .join(' ', map { ' -vf '.$_ } @transforms)
+       ." -s $video_size "
+
       .$Config{ZM_FFMPEG_OUTPUT_OPTIONS}
     ." '$video_file' > ffmpeg.log 2>&1"
       ;
@@ -355,7 +365,7 @@ sub GenerateVideo {
 # commits unless we started the transaction.
 
 sub delete {
-  my $event = $_[0];
+  my $event = shift;
 
   if ( !$event->canEdit() ) {
     Warning('No permission to delete event.');
@@ -393,6 +403,11 @@ sub delete {
       $ZoneMinder::Database::dbh->commit() if ! $in_transaction;
       return;
     }
+    ZoneMinder::Database::zmDbDo('DELETE FROM Event_Data WHERE EventId=?', $$event{Id});
+    if ( $ZoneMinder::Database::dbh->errstr() ) {
+      $ZoneMinder::Database::dbh->commit() if ! $in_transaction;
+      return;
+    }
     ZoneMinder::Database::zmDbDo('DELETE FROM Frames WHERE EventId=?', $$event{Id});
     if ( $ZoneMinder::Database::dbh->errstr() ) {
       $ZoneMinder::Database::dbh->commit() if ! $in_transaction;
@@ -402,6 +417,9 @@ sub delete {
     # Do it individually to avoid locking up the table for new events
     ZoneMinder::Database::zmDbDo('DELETE FROM Events WHERE Id=?', $$event{Id});
     $ZoneMinder::Database::dbh->commit() if ! $in_transaction;
+
+    my $storage = $event->Storage();
+    $storage->save({DiskSpace=>$storage->DiskSpace()-$event->DiskSpace()}) if $event->DiskSpace() and $storage->Id();
   }
 
   if ( ( $in_zmaudit or (!$Config{ZM_OPT_FAST_DELETE})) and $event->Storage()->DoDelete() ) {
@@ -441,37 +459,7 @@ sub delete_files {
       ( $storage_path ) = ( $storage_path =~ /^(.*)$/ ); # De-taint
       ( $event_path ) = ( $event_path =~ /^(.*)$/ ); # De-taint
 
-      my $deleted = 0;
-      if ( $$Storage{Type} and ( $$Storage{Type} eq 's3fs' ) ) {
-        my $url = $$Storage{Url};
-        $url =~ s/^(s3|s3fs):\/\///ig;
-        my ( $aws_id, $aws_secret, $aws_host, $aws_bucket, $subpath ) = ( $url =~ /^\s*([^:]+):([^@]+)@([^\/]*)\/([^\/]+)(\/.+)?\s*$/ );
-        Debug("S3 url parsed to id:$aws_id secret:$aws_secret host:$aws_host, bucket:$aws_bucket, subpath:$subpath\n from $url");
-        eval {
-          require Net::Amazon::S3;
-          my $s3 = Net::Amazon::S3->new( {
-              aws_access_key_id     => $aws_id,
-              aws_secret_access_key => $aws_secret,
-              ( $aws_host ? ( host => $aws_host ) : () ),
-              authorization_method => 'Net::Amazon::S3::Signature::V4',
-            });
-          my $bucket = $s3->bucket($aws_bucket);
-          if ( ! $bucket ) {
-            Error("S3 bucket $bucket not found.");
-            die;
-          }
-          if ( $bucket->delete_key($subpath.$event_path) ) {
-            $deleted = 1;
-          } else {
-            Error('Failed to delete from S3:'.$s3->err . ': ' . $s3->errstr);
-          }
-        };
-        Error($@) if $@;
-      } # end if s3fs
-      if ( !$deleted ) {
-        my $command = "/bin/rm -rf $storage_path/$event_path";
-        ZoneMinder::General::executeShellCommand($command);
-      }
+      $Storage->delete_path($event_path);
     } else {
       Error('No event path in delete files. ' . $event->to_string());
     } # end if event_path
@@ -485,33 +473,74 @@ sub delete_files {
       }
     } # end if Scheme eq Deep
 
-    # Now check for empty directories and delete them.
-    my @path_parts = split('/', $event_path);
-    pop @path_parts;
-    # Guaranteed the first part is the monitor id
-    Debug("Initial path_parts: @path_parts");
-    while ( @path_parts > 1 ) {
-      my $path = join('/', $storage_path, @path_parts);
-      my $dh;
-      if ( !opendir($dh, $path) ) {
-        Warning("Fail to open $path");
-        last;
-      }
-      my @dir =  readdir($dh);
-      closedir($dh);
-      if ( scalar(grep { $_ ne '.' and $_ ne '..' } @dir) == 0 ) {
-        Debug("Removing empty dir at $path");
-        if ( !rmdir $path ) {
-          Warning("Fail to rmdir $path: $!");
+    if ($Storage->Type() ne 's3fs') {
+      # Now check for empty directories and delete them.
+      my @path_parts = split('/', $event_path);
+      pop @path_parts;
+      # Guaranteed the first part is the monitor id
+      Debug("Initial path_parts: @path_parts");
+      while ( @path_parts > 1 ) {
+        my $path = join('/', $storage_path, @path_parts);
+        my $dh;
+        if ( !opendir($dh, $path) ) {
+          Warning("Fail to open $path");
           last;
         }
-      } else {
-        Debug("Dir $path is not empty @dir");
-        last;
-      }
-      pop @path_parts;
-    } # end while path_parts
+        my @dir =  readdir($dh);
+        closedir($dh);
+        if ( scalar(grep { $_ ne '.' and $_ ne '..' } @dir) == 0 ) {
+          Debug("Removing empty dir at $path");
+          if ( !rmdir $path ) {
+            Warning("Fail to rmdir $path: $!");
+            last;
+          }
+        } else {
+          Debug(4, "Dir $path is not empty @dir");
+          last;
+        }
+        pop @path_parts;
+      } # end while path_parts
+    } # end if not s3fs
+  } # end foreach Storage
+} # end sub delete_files
 
+sub delete_analysis_jpegs {
+  my $event = shift;
+
+  if ( !$event->canEdit() ) {
+    Warning('No permission to delete event.');
+    return 'No permission to delete event.';
+  }
+
+  foreach my $Storage (
+    @_ ? ($_[0]) : (
+      new ZoneMinder::Storage($$event{StorageId}),
+      ( $$event{SecondaryStorageId} ? new ZoneMinder::Storage($$event{SecondaryStorageId}) : () ),
+    ) ) {
+    my $storage_path = $Storage->Path();
+
+    if ( !$storage_path ) {
+      Error("Empty storage path when deleting files for event $$event{Id} with storage id $$event{StorageId}");
+      return;
+    }
+
+    if ( !$$event{MonitorId} ) {
+      Error("No monitor id assigned to event $$event{Id}");
+      return;
+    }
+    my $event_path = $event->RelativePath();
+    Debug("Deleting analysis jpegs for Event $$event{Id} from $storage_path///$event_path, scheme is $$event{Scheme}.");
+    if ( $event_path ) {
+      ( $storage_path ) = ( $storage_path =~ /^(.*)$/ ); # De-taint
+      ( $event_path ) = ( $event_path =~ /^(.*)$/ ); # De-taint
+      my @files = glob("$storage_path/$event_path/*analyse.jpg");
+      Debug(@files . ' analysis jpegs found to delete');
+      foreach my $file (@files) {
+        $Storage->delete_path($event_path.'/'.$file);
+      }
+    } else {
+      Error('No event path in delete files. ' . $event->to_string());
+    } # end if event_path
   } # end foreach Storage
 } # end sub delete_files
 
@@ -599,7 +628,7 @@ sub CopyTo {
   # First determine if we can move it to the dest.
   # We do this before bothering to lock the event
   my ( $NewPath ) = ( $NewStorage->Path() =~ /^(.*)$/ ); # De-taint
-  if ( ! $$NewStorage{Id} ) {
+  if ( ! looks_like_number($$NewStorage{Id}) ) {
     return 'New storage does not have an id.  Moving will not happen.';
   } elsif ( $$NewStorage{Id} == $$self{StorageId} ) {
     return 'Event is already located at ' . $NewPath;
@@ -626,64 +655,44 @@ sub CopyTo {
 	$NewPath .= '/'.$self->RelativePath();
 	($NewPath) = ( $NewPath =~ /^(.*)$/ ); # De-taint
   if ( $NewPath eq $OldPath ) {
-    return "New path and old path are the same! $NewPath";
+    $ZoneMinder::Database::dbh->commit();
+    return "New path and old path are the same! $OldPath $NewPath";
   }
   Debug("Copying event $$self{Id} from $OldPath to $NewPath");
 
   my $moved = 0;
 
-  if ( $$NewStorage{Type} eq 's3fs' ) {
-    if ( $$NewStorage{Url} ) {
-      my $url = $$NewStorage{Url};
-      $url =~ s/^(s3|s3fs):\/\///ig;
-      my ( $aws_id, $aws_secret, $aws_host, $aws_bucket, $subpath ) = ( $url =~ /^\s*([^:]+):([^@]+)@([^\/]*)\/([^\/]+)(\/.+)?\s*$/ );
-      Debug("S3 url parsed to id:$aws_id secret:$aws_secret host:$aws_host, bucket:$aws_bucket, subpath:$subpath\n from $url");
-      if ( $aws_id and $aws_secret and $aws_host and $aws_bucket ) {
-        eval {
-          require Net::Amazon::S3;
-          require File::Slurp;
-          my $s3 = Net::Amazon::S3->new( {
-              aws_access_key_id     => $aws_id,
-              aws_secret_access_key => $aws_secret,
-              ( $aws_host ? ( host => $aws_host ) : () ),
-              authorization_method => 'Net::Amazon::S3::Signature::V4',
-            });
-          my $bucket = $s3->bucket($aws_bucket);
-          if ( !$bucket ) {
-            Error("S3 bucket $bucket not found.");
-            die;
+  if ($$NewStorage{Type} eq 's3fs') {
+    my $s3 = $NewStorage->s3();
+    my $bucket = $NewStorage->bucket();
+    if ($s3 and $bucket) {
+      my $event_path = $NewStorage->aws_subpath().$self->RelativePath();
+      my @files = glob("$OldPath/*");
+      Debug("Files to move @files");
+      eval {
+        foreach my $file (@files) {
+          next if $file =~ /^\./;
+          ($file) = ($file =~ /^(.*)$/); # De-taint
+          my $starttime = [gettimeofday];
+          Debug("Moving file $file to $NewPath");
+          my $size = -s $file;
+          if (!$size) {
+            Info('Not moving file with 0 size');
+          }
+          my $filename = $event_path.'/'.File::Basename::basename($file);
+          if (!$bucket->add_key_filename($filename, $file)) {
+            die "Unable to add key for $filename " . $s3->err . ': '. $s3->errstr;
           }
 
-          my $event_path = $subpath.$self->RelativePath();
+          my $duration = tv_interval($starttime);
+          Debug('PUT to S3 ' . Number::Bytes::Human::format_bytes($size) . " in $duration seconds = " . Number::Bytes::Human::format_bytes($duration?$size/$duration:$size) . '/sec');
+        } # end foreach file.
 
-          my @files = glob("$OldPath/*");
-          Debug("Files to move @files");
-          foreach my $file (@files) {
-            next if $file =~ /^\./;
-            ($file) = ($file =~ /^(.*)$/); # De-taint
-            my $starttime = [gettimeofday];
-            Debug("Moving file $file to $NewPath");
-            my $size = -s $file;
-            if (!$size) {
-              Info('Not moving file with 0 size');
-            }
-            my $filename = $event_path.'/'.File::Basename::basename($file);
-            if (!$bucket->add_key_filename($filename, $file)) {
-              die "Unable to add key for $filename " . $s3->err . ': '. $s3->errstr;
-            }
-
-            my $duration = tv_interval($starttime);
-            Debug('PUT to S3 ' . Number::Bytes::Human::format_bytes($size) . " in $duration seconds = " . Number::Bytes::Human::format_bytes($duration?$size/$duration:$size) . '/sec');
-          } # end foreach file.
-
-          $moved = 1;
-        };
-        Error($@) if $@;
-      } else {
-        Error('Unable to parse S3 Url into it\'s component parts.');
-      }
-    } # end if Url
-  } # end if s3
+        $moved = 1;
+      };
+      Error($@) if $@;
+    } # end if s3
+  } # end if s3f3
 
   my $error = '';
   if (!$moved) {
@@ -700,12 +709,16 @@ sub CopyTo {
       }
     }
     return $error if $error;
-    my @files = glob("$OldPath/*");
+    Debug("Made new path at $NewPath, starting to glob");
+    opendir(my $dh, $OldPath) || return "Failed to open $OldPath";
+    my @files = readdir($dh);
     return 'No files to move.' if !@files;
 
     for my $file (@files) {
       next if $file =~ /^\./;
       ($file) = ($file =~ /^(.*)$/); # De-taint
+      $file = $OldPath.'/'.$file;
+      next if !-f $file;
       my $starttime = [gettimeofday];
       my $size = -s $file;
       if (!File::Copy::copy($file, $NewPath)) {
@@ -730,24 +743,47 @@ sub MoveTo {
 
   my $was_in_transaction = !$ZoneMinder::Database::dbh->{AutoCommit};
   $ZoneMinder::Database::dbh->begin_work() if !$was_in_transaction;
-  $self->lock_and_load(); # The fact that we are in a transaction might not imply locking
+  if (!$self->lock_and_load()) {
+    Warning('Unable to lock event record '.$$self{Id}); # The fact that we are in a transaction might not imply locking
+    $ZoneMinder::Database::dbh->commit() if !$was_in_transaction;
+    return 'Unable to lock event record';
+  }
 
   my $OldStorage = $self->Storage(undef);
 
-  my $error = $self->CopyTo($NewStorage);
-  if ($error) {
-    $ZoneMinder::Database::dbh->commit() if !$was_in_transaction;
-    return $error;
+  # In strange situations where commits don't happen, the files can be moved but the db hasn't been updated.
+  # So here's a special case test to fix that.
+  my ( $SrcPath ) = ( $self->Path(undef) =~ /^(.*)$/ ); # De-taint
+  my $NewPath = $NewStorage->Path().'/'.$self->RelativePath();
+  $NewPath =~ /^(.*)$/; # De-taint
+
+  my $error = '';
+  if ((! -e $SrcPath) and -e $NewPath) {
+    Warning("Event $$self{Id} has already been moved from $$OldStorage{Id} $SrcPath, just updating the event record in db to $$NewStorage{Id} $NewPath.");
+  } else {
+    $error = $self->CopyTo($NewStorage);
+    if ($error) {
+      $ZoneMinder::Database::dbh->commit() if !$was_in_transaction;
+      return $error;
+    }
   }
 
+  my $old_diskspace = $self->DiskSpace();
+  my $new_diskspace = $self->DiskSpace(undef);
+
   # Succeeded in copying all files, so we may now update the Event.
-  $$self{StorageId} = $$NewStorage{Id};
   $self->Storage($NewStorage);
   $error .= $self->save();
-
   # Going to leave it to upper layer as to whether we rollback or not
+  if ($error) {
+    $ZoneMinder::Database::dbh->rollback() if !$was_in_transaction;
+    return $error;
+  }
   $ZoneMinder::Database::dbh->commit() if !$was_in_transaction;
-  return $error if $error;
+
+  # Update storage diskspace.  The triggers no longer do this. This is ... less important so do it outside the transaction
+  $OldStorage->save({DiskSpace => $OldStorage->DiskSpace()-$old_diskspace}) if $old_diskspace and $OldStorage->Id();
+  $NewStorage->save({DiskSpace => $NewStorage->DiskSpace()+$new_diskspace}) if $new_diskspace and $NewStorage->Id();
 
   $self->delete_files($OldStorage);
   return $error;
@@ -759,6 +795,9 @@ sub recover_timestamps {
   my ( $Event, $path ) = @_;
   $path = $Event->Path() if ! $path;
 
+  # Get timestamp of the dir.  Any files older than this will override this as the starttime
+  my $starttime = (stat($path))[9];
+
   if ( !opendir(DIR, $path) ) {
     Error("Can't open directory '$path': $!");
     return;
@@ -767,7 +806,7 @@ sub recover_timestamps {
   Debug('Have ' . @contents . ' files in '.$path);
   closedir(DIR);
 
-  my @mp4_files = grep(/^\d+\-video\.mp4$/, @contents);
+  my @mp4_files = grep(/^\d+\-video\.\w+\.mp4$/, @contents);
   if ( @mp4_files ) {
     $$Event{DefaultVideo} = $mp4_files[0];
   }
@@ -786,6 +825,7 @@ sub recover_timestamps {
     my $first_file = "$path/$capture_jpgs[0]";
     ( $first_file ) = $first_file =~ /^(.*)$/;
     my $first_timestamp = (stat($first_file))[9];
+    $starttime = $first_timestamp if $first_timestamp < $starttime;
 
     my $last_file = $path.'/'.$capture_jpgs[@capture_jpgs-1];
     ( $last_file ) = $last_file =~ /^(.*)$/;
@@ -826,6 +866,7 @@ sub recover_timestamps {
     ( $file ) = $file =~ /^(.*)$/;
 
     my $first_timestamp = (stat($file))[9];
+    $starttime = $first_timestamp if $first_timestamp < $starttime;
     my $output = `ffprobe $file 2>&1`;
     my ($duration) = $output =~ /Duration: [:\.0-9]+/gm;
     Debug("From mp4 have duration $duration, start: $first_timestamp");
@@ -843,8 +884,41 @@ sub recover_timestamps {
   if ( @mp4_files ) {
     $Event->DefaultVideo($mp4_files[0]);
   }
+  $Event->StartDateTime( Date::Format::time2str('%Y-%m-%d %H:%M:%S', $starttime) );
 }
 
+sub guess_EndDateTime {
+  my $event = shift;
+  if (!$$event{EndDateTime}) {
+    if ($$event{Length}) {
+      my $startdatetime = Date::Parse::str2time( $_[0]{StartDateTime} );
+      $$event{EndDateTime} = Date::Format::time2str('%Y-%m-%d %H:%M:%S', $startdatetime + $$event{Length});
+    } else {
+      $$event{EndDateTime} = Date::Format::time2str('%Y-%m-%d %H:%M:%S', Date::Parse::str2time( $_[0]{StartDateTime} )+1);
+    }
+  }
+  return $$event{EndDateTime};
+}
+
+sub fix_DefaultVideo {
+  my $event = shift;
+  if (!$$event{DefaultVideo} or $$event{DefaultVideo} eq 'incomplete.mp4' or ! -e $event->Path().'/'.$$event{DefaultVideo}) {
+    my $path = $event->Path();
+    if ( !opendir(DIR, $path) ) {
+      Error("Can't open directory '$path': $!");
+      return;
+    }
+    my @contents = readdir(DIR);
+    Debug('Have ' . @contents . ' files in '.$path);
+    closedir(DIR);
+
+    my @mp4_files = grep(/^\d+\-video\.\w+\.mp4$/, @contents);
+    if ( @mp4_files ) {
+      $$event{DefaultVideo} = $mp4_files[0];
+    }
+  }
+} # end sub fix_DefaultVideo
+
 sub files {
 	my $self = shift;
 
@@ -897,6 +971,65 @@ sub Monitor {
   return $$self{Monitor};
 }
 
+sub Close {
+  my $self = shift;
+  my $tag = @_ ? shift : '(r)';
+  my $text = @_ ? shift : 'Recovered.';
+
+  my $FrameDataSql = '
+SELECT
+  max(`TimeStamp`) AS `EndDateTime`,
+  unix_timestamp(max(`TimeStamp`)) AS `EndTimeStamp`,
+  max(`FrameId`) AS `Frames`,
+  count(if(`Score`>0,1,NULL)) AS `AlarmFrames`,
+  sum(`Score`) AS `TotScore`,
+  max(`Score`) AS `MaxScore`
+FROM `Frames` WHERE `EventId`=?';
+  my $frame = ZoneMinder::Database::zmDbFetchOne($FrameDataSql, $self->{Id});
+  if (!$frame) {
+    return 'Unable to retrieve frame data.';
+  }
+  if (!$frame->{EndDateTime}) {
+    return 'Unable to retrieve EndDateTime from Frames Table.';
+  }
+  return $self->save({
+      Name => sprintf('%s%d%s', $self->Monitor()->EventPrefix(), $self->{Id}, $tag),
+      EndDateTime => $frame->{EndDateTime},
+      Length => $frame->{EndTimeStamp} - $self->Time(),
+      Frames => $frame->{Frames},
+      AlarmFrames => $frame->{AlarmFrames},
+      TotScore => $frame->{TotScore},
+      AvgScore => ($frame->{AlarmFrames} ? int($frame->{TotScore} / $frame->{AlarmFrames}) : 0),
+      MaxScore => $frame->{MaxScore},
+      Notes => $self->Notes() . ' ' . $text,
+    });
+} # end sub Close
+
+sub Event_Tags {
+  my $self = shift;
+  $$self{Event_Tags} = shift if @_;
+  if (!$$self{Event_Tags}) {
+    $$self{Event_Tags} = [ ZoneMinder::Event_Tag->find(EventId=>$$self{Id}) ];
+  }
+  return wantarray ? @{$$self{Event_Tags}} : $$self{Event_Tags};
+}
+
+sub Tags {
+  my $self = shift;
+  $$self{Tags} = shift if @_;
+
+  if (!$$self{Tags}) {
+    $$self{Tags} = [ map { $_->Tag() } $self->Event_Tags() ];
+  }
+  return wantarray ? @{$$self{Tags}} : $$self{Tags};
+}
+
+sub tags {
+  my $self = shift;
+  my @tags = map { $_->Name() } $self->Tags();
+  return wantarray ? @tags : \@tags;
+}
+
 1;
 __END__
 
