@@ -3,34 +3,31 @@
 #include "zm_ffmpeg.h"
 #include "zm_logger.h"
 
-FFmpeg_Input::FFmpeg_Input() {
-  input_format_context = nullptr;
-  video_stream_id = -1;
-  audio_stream_id = -1;
+FFmpeg_Input::FFmpeg_Input() :
+  streams(nullptr),
+  video_stream_id(-1),
+  audio_stream_id(-1),
+  input_format_context(nullptr),
+  last_seek_request(-1),
+  hw_device_ctx(nullptr)
+{
   FFMPEGInit();
-  streams = nullptr;
-  frame = nullptr;
-	last_seek_request = -1;
 }
 
 FFmpeg_Input::~FFmpeg_Input() {
   if (input_format_context) {
     Close();
   }
-  if ( frame ) {
-    av_frame_free(&frame);
-    frame = nullptr;
-  }
 }  // end ~FFmpeg_Input()
 
 /* Takes streams provided from elsewhere.  They might not come from the same source
  * but we will treat them as if they are.  */
 int FFmpeg_Input::Open(
-    const AVStream * video_in_stream,
-    const AVCodecContext * video_in_ctx,
-    const AVStream * audio_in_stream,
-    const AVCodecContext * audio_in_ctx
-    ) {
+  const AVStream * video_in_stream,
+  const AVCodecContext * video_in_ctx,
+  const AVStream * audio_in_stream,
+  const AVCodecContext * audio_in_ctx
+) {
   int max_stream_index = video_stream_id = video_in_stream->index;
 
   if (audio_in_stream) {
@@ -48,7 +45,7 @@ int FFmpeg_Input::Open(const char *filepath) {
   error = avformat_open_input(&input_format_context, filepath, nullptr, nullptr);
   if ( error < 0 ) {
     Error("Could not open input file '%s' (error '%s')",
-        filepath, av_make_error_string(error).c_str());
+          filepath, av_make_error_string(error).c_str());
     input_format_context = nullptr;
     return error;
   }
@@ -56,9 +53,9 @@ int FFmpeg_Input::Open(const char *filepath) {
   /** Get information on the input file (number of streams etc.). */
   if ( (error = avformat_find_stream_info(input_format_context, nullptr)) < 0 ) {
     Error(
-        "Could not open find stream info (error '%s')",
-        av_make_error_string(error).c_str()
-        );
+      "Could not open find stream info (error '%s')",
+      av_make_error_string(error).c_str()
+    );
     avformat_close_input(&input_format_context);
     return error;
   }
@@ -66,7 +63,10 @@ int FFmpeg_Input::Open(const char *filepath) {
   streams = new stream[input_format_context->nb_streams];
   Debug(2, "Have %d streams", input_format_context->nb_streams);
 
+
   for (unsigned int i = 0; i < input_format_context->nb_streams; i += 1) {
+    av_seek_frame(input_format_context, i, 0, AVSEEK_FLAG_FRAME);
+
     if (is_video_stream(input_format_context->streams[i])) {
       zm_dump_stream_format(input_format_context, i, 0, 0);
       if (video_stream_id == -1) {
@@ -88,45 +88,61 @@ int FFmpeg_Input::Open(const char *filepath) {
 
     streams[i].frame_count = 0;
 
-    if (!(streams[i].codec = avcodec_find_decoder(
-#if LIBAVCODEC_VERSION_CHECK(57, 64, 0, 64, 0)
-            input_format_context->streams[i]->codecpar->codec_id
-#else
-            input_format_context->streams[i]->codec->codec_id
-#endif
-            ))) {
-      Error("Could not find input codec");
-      avformat_close_input(&input_format_context);
-      return AVERROR_EXIT;
-    } else {
-      Debug(1, "Using codec (%s) for stream %d", streams[i].codec->name, i);
+    streams[i].context = nullptr;
+    std::list<const CodecData *>codec_data = get_decoder_data(input_format_context->streams[i]->codecpar->codec_id, "auto");
+    for (auto it = codec_data.begin(); it != codec_data.end(); it ++) {
+      const CodecData *chosen_codec_data = *it;
+      Debug(1, "Found codec %s", chosen_codec_data->codec_name);
+
+      streams[i].codec = avcodec_find_decoder_by_name(chosen_codec_data->codec_name);
+
+      streams[i].context = avcodec_alloc_context3(streams[i].codec);
+      avcodec_parameters_to_context(streams[i].context, input_format_context->streams[i]->codecpar);
+      if (setup_hwaccel(streams[i].context,
+            chosen_codec_data, hw_device_ctx, "", input_format_context->streams[i]->codecpar->width,
+            input_format_context->streams[i]->codecpar->height
+            )) {
+        continue;
+      }
+
+      zm_dump_codec(streams[i].context);
+
+      error = avcodec_open2(streams[i].context, streams[i].codec, nullptr);
+      if (error < 0) {
+        Error("Could not open input codec (error '%s')",
+            av_make_error_string(error).c_str());
+        avcodec_free_context(&streams[i].context);
+        streams[i].context = nullptr;
+        continue;
+      }
+    } // end foreach codec_data
+ 
+    if (!streams[i].context) {
+      if ((streams[i].codec = avcodec_find_decoder(input_format_context->streams[i]->codecpar->codec_id))) {
+        Debug(1, "Using codec (%s) for stream %d", streams[i].codec->name, i);
+        streams[i].context = avcodec_alloc_context3(streams[i].codec);
+        avcodec_parameters_to_context(streams[i].context, input_format_context->streams[i]->codecpar);
+
+        zm_dump_codec(streams[i].context);
+
+        error = avcodec_open2(streams[i].context, streams[i].codec, nullptr);
+        if (error < 0) {
+          Error("Could not open input codec (error '%s')", av_make_error_string(error).c_str());
+          avcodec_free_context(&streams[i].context);
+          streams[i].context = nullptr;
+        }
+      }
     }
 
-#if LIBAVCODEC_VERSION_CHECK(57, 64, 0, 64, 0)
-    streams[i].context = avcodec_alloc_context3(nullptr);
-    avcodec_parameters_to_context(streams[i].context, input_format_context->streams[i]->codecpar);
-#else
-    streams[i].context = input_format_context->streams[i]->codec;
-#endif
-    // Some codecs will change the time base others might not. h265 seems to not.  So let's set a sane value
-    streams[i].context->time_base.num = 1;
-    streams[i].context->time_base.den = 90000;
-    zm_dump_codec(streams[i].context);
-
-    error = avcodec_open2(streams[i].context, streams[i].codec, nullptr);
-    if (error < 0) {
-      Error("Could not open input codec (error '%s')",
-          av_make_error_string(error).c_str());
-#if LIBAVCODEC_VERSION_CHECK(57, 64, 0, 64, 0)
-      avcodec_free_context(&streams[i].context);
-#endif
-      avformat_close_input(&input_format_context);
-      input_format_context = nullptr;
+    if (!streams[i].context) {
+      //avformat_close_input(&input_format_context);
+      //input_format_context = nullptr;
+      continue;
       return error;
     }
     zm_dump_codec(streams[i].context);
     if (!(streams[i].context->time_base.num && streams[i].context->time_base.den)) {
-      Warning("Setting to default time base 1/90000");
+      Debug(1, "Setting to default time base");
       streams[i].context->time_base.num = 1;
       streams[i].context->time_base.den = 90000;
     }
@@ -143,30 +159,28 @@ int FFmpeg_Input::Open(const char *filepath) {
 int FFmpeg_Input::Close( ) {
   if (streams) {
     for (unsigned int i = 0; i < input_format_context->nb_streams; i += 1) {
-#if LIBAVCODEC_VERSION_CHECK(57, 64, 0, 64, 0)
+      //avcodec_close(streams[i].context);
       avcodec_free_context(&streams[i].context);
       streams[i].context = nullptr;
-#else
-      avcodec_close(streams[i].context);
-#endif
     }
     delete[] streams;
     streams = nullptr;
   }
 
   if (input_format_context) {
-#if !LIBAVFORMAT_VERSION_CHECK(53, 17, 0, 25, 0)
-    av_close_input_file(input_format_context);
-#else
     avformat_close_input(&input_format_context);
-#endif
     input_format_context = nullptr;
   }
   return 1;
 } // end int FFmpeg_Input::Close()
 
 AVFrame *FFmpeg_Input::get_frame(int stream_id) {
-  int frameComplete = false;
+  if (!streams[stream_id].context) {
+    Error("No context for stream %d", stream_id);
+    return nullptr;
+  }
+
+  bool frameComplete = false;
   av_packet_ptr packet{av_packet_alloc()};
 
   if (!packet) {
@@ -174,80 +188,65 @@ AVFrame *FFmpeg_Input::get_frame(int stream_id) {
     return nullptr;
   }
 
+
   while (!frameComplete) {
     int ret = av_read_frame(input_format_context, packet.get());
     if (ret < 0) {
       if (
-          // Check if EOF.
-          (ret == AVERROR_EOF || (input_format_context->pb && input_format_context->pb->eof_reached)) ||
-          // Check for Connection failure.
-          (ret == -110)
-         ) {
+        // Check if EOF.
+        (ret == AVERROR_EOF || (input_format_context->pb && input_format_context->pb->eof_reached)) ||
+        // Check for Connection failure.
+        (ret == -110)
+      ) {
         Info("av_read_frame returned %s.", av_make_error_string(ret).c_str());
         return nullptr;
       }
       Error("Unable to read packet from stream %d: error %d \"%s\".",
-          packet->stream_index, ret, av_make_error_string(ret).c_str());
+            packet->stream_index, ret, av_make_error_string(ret).c_str());
       return nullptr;
     }
     ZM_DUMP_STREAM_PACKET(input_format_context->streams[packet->stream_index], packet, "Received packet");
 
     av_packet_guard pkt_guard{packet};
+
     if ((stream_id >= 0) && (packet->stream_index != stream_id)) {
-      Debug(1,"Packet is not for our stream (%d)", packet->stream_index );
+      Debug(4, "Packet is not for our stream (%d)", packet->stream_index );
       continue;
     }
 
     AVCodecContext *context = streams[packet->stream_index].context;
 
-    if ( frame ) {
-      av_frame_free(&frame);
-      frame = zm_av_frame_alloc();
-    } else {
-      frame = zm_av_frame_alloc();
+    frame = av_frame_ptr{zm_av_frame_alloc()};
+    if (!frame) {
+      Error("Unable to allocate frame.");
+      return nullptr;
     }
-    ret = zm_send_packet_receive_frame(context, frame, *packet);
+    ret = zm_send_packet_receive_frame(context, frame.get(), *packet);
     if ( ret < 0 ) {
       Error("Unable to decode frame at frame %d: %d %s, continuing",
-          streams[packet->stream_index].frame_count, ret, av_make_error_string(ret).c_str());
-      av_frame_free(&frame);
+            streams[packet->stream_index].frame_count, ret, av_make_error_string(ret).c_str());
+      frame = nullptr;
       continue;
     } else {
       if (is_video_stream(input_format_context->streams[packet->stream_index])) {
-        zm_dump_video_frame(frame, "resulting video frame");
+        zm_dump_video_frame(frame.get(), "resulting video frame");
       } else {
-        zm_dump_frame(frame, "resulting frame");
-      }
-      if (frame->pts == AV_NOPTS_VALUE) {
-#if LIBAVCODEC_VERSION_CHECK(57, 64, 0, 64, 0)
-        frame->pts = frame->pkt_dts;
-#else
-        frame->pts = frame->pkt_pts;
-#endif
+        zm_dump_frame(frame.get(), "resulting frame");
       }
     }
 
     frameComplete = true;
 
-    if (context->time_base.num && context->time_base.den) {
-    // Convert timestamps to stream timebase instead of codec timebase
-    frame->pts = av_rescale_q(frame->pts,
-        context->time_base,
-        input_format_context->streams[stream_id]->time_base
-        );
-    } else {
-      Warning("No timebase set in context!");
-    }
     if (is_video_stream(input_format_context->streams[packet->stream_index])) {
-      zm_dump_video_frame(frame, "resulting video frame");
+      zm_dump_video_frame(frame.get(), "resulting video frame");
     } else {
-      zm_dump_frame(frame, "resulting frame");
+      zm_dump_frame(frame.get(), "resulting frame");
     }
-
   } // end while !frameComplete
-  return frame;
+  return frame.get();
 }  // end AVFrame *FFmpeg_Input::get_frame
 
+/* at is FPSeconds */
 AVFrame *FFmpeg_Input::get_frame(int stream_id, double at) {
   Debug(1, "Getting frame from stream %d at %f", stream_id, at);
 
@@ -263,7 +262,7 @@ AVFrame *FFmpeg_Input::get_frame(int stream_id, double at) {
     ret = av_seek_frame(input_format_context, stream_id, seek_target, AVSEEK_FLAG_FRAME);
     if (ret < 0) {
       Error("Unable to seek in stream");
-      return nullptr;
+      //return nullptr;
     }
     // Have to grab a frame to update our current frame to know where we are
     get_frame(stream_id);
@@ -274,18 +273,18 @@ AVFrame *FFmpeg_Input::get_frame(int stream_id, double at) {
     }
   }  // end if ! frame
 
-  if ( 
-			(last_seek_request >= 0)
-			&&
-			(last_seek_request > seek_target) 
-			&&
-			(frame->pts > seek_target)
-		 ) {
+  if (
+    (last_seek_request >= 0)
+    &&
+    (last_seek_request > seek_target)
+    &&
+    (frame->pts > seek_target)
+  ) {
     zm_dump_frame(frame, "frame->pts > seek_target, seek backwards");
-  // our frame must be beyond our seek target. so go backwards to before it
-    if (( ret = av_seek_frame(input_format_context, stream_id, seek_target, 
-            AVSEEK_FLAG_BACKWARD | AVSEEK_FLAG_FRAME
-            ) ) < 0) {
+    // our frame must be beyond our seek target. so go backwards to before it
+    if (( ret = av_seek_frame(input_format_context, stream_id, seek_target,
+                              AVSEEK_FLAG_BACKWARD | AVSEEK_FLAG_FRAME
+                             ) ) < 0) {
       Error("Unable to seek in stream %d", ret);
       return nullptr;
     }
@@ -298,45 +297,54 @@ AVFrame *FFmpeg_Input::get_frame(int stream_id, double at) {
     }
   } else if ( last_seek_request == seek_target ) {
     // paused case, sending keepalives
-    return frame;
+    return frame.get();
   } // end if frame->pts > seek_target
 
-	last_seek_request = seek_target;
+  last_seek_request = seek_target;
 
   // Normally it is likely just the next packet. Need a heuristic for seeking, something like duration * keyframe interval
 #if LIBAVCODEC_VERSION_CHECK(60, 3, 0, 3, 0)
-  if (frame->pts + 10*frame->duration < seek_target)
+  if (frame->pts + 10*input_format_context->streams[stream_id]->time_base.den * frame->duration < seek_target)
 #else
-  if (frame->pts + 10*frame->pkt_duration < seek_target)
+  if (frame->pts + 10*input_format_context->streams[stream_id]->time_base.den * frame->pkt_duration < seek_target)
 #endif
   {
-    Debug(1, "Jumping ahead");
-    if (( ret = av_seek_frame(input_format_context, stream_id, seek_target,
-            AVSEEK_FLAG_FRAME
-            ) ) < 0) {
+    Debug(1, "Jumping ahead to %" PRId64, seek_target);
+    if (( ret = av_seek_frame(input_format_context, stream_id, seek_target, AVSEEK_FLAG_FRAME) ) < 0) {
       Error("Unable to seek in stream %d", ret);
       return nullptr;
     }
     // Have to grab a frame to update our current frame to know where we are
     get_frame(stream_id);
+    zm_dump_frame(frame, "got");
+    if (frame->pts > seek_target) {
+      if (( ret = av_seek_frame(input_format_context, stream_id, seek_target,
+                                AVSEEK_FLAG_FRAME|AVSEEK_FLAG_BACKWARD
+                               ) ) < 0) {
+        Error("Unable to seek in stream %d", ret);
+        return nullptr;
+      }
+      get_frame(stream_id);
+      zm_dump_frame(frame, "Had to seek backwards");
+    }
   }
   // Seeking seems to typically seek to a keyframe, so then we have to decode until we get the frame we want.
   if (frame->pts <= seek_target) {
     Debug(1, "Frame pts %" PRId64 " + duration %" PRId64 "= %" PRId64 " <=? %" PRId64,
-        frame->pts,
+          frame->pts,
 #if LIBAVCODEC_VERSION_CHECK(60, 3, 0, 3, 0)
-        frame->duration, frame->pts + frame->duration,
+          frame->duration, frame->pts + frame->duration,
 #else
-        frame->pkt_duration, frame->pts + frame->pkt_duration,
+          frame->pkt_duration, frame->pts + frame->pkt_duration,
 #endif
-        seek_target);
+          seek_target);
     while (frame && (frame->pts +
 #if LIBAVCODEC_VERSION_CHECK(60, 3, 0, 3, 0)
-          frame->duration
+                     frame->duration
 #else
-          frame->pkt_duration
+                     frame->pkt_duration
 #endif
-          < seek_target)) {
+                     < seek_target)) {
       if (is_video_stream(input_format_context->streams[stream_id])) {
         zm_dump_video_frame(frame, "pts <= seek_target");
       } else {
@@ -344,12 +352,11 @@ AVFrame *FFmpeg_Input::get_frame(int stream_id, double at) {
       }
       if (!get_frame(stream_id)) {
         Warning("Got no frame. returning nothing");
-        return frame;
+        return frame.get();
       }
     }
-    zm_dump_frame(frame, "frame->pts <= seek_target, got");
-    return frame;
+    return frame.get();
   }
 
   return get_frame(stream_id);
-}  // end AVFrame *FFmpeg_Input::get_frame( int stream_id, struct timeval at)
+}
