@@ -21,22 +21,124 @@
 
 #include "zm_box.h"
 #include "zm_monitor.h"
+#include "zm_signal.h"
+
 #include <cmath>
 #include <sys/file.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#include <fcntl.h>
+
+constexpr Seconds StreamBase::MAX_STREAM_DELAY;
+constexpr Milliseconds StreamBase::MAX_SLEEP;
 
 StreamBase::~StreamBase() {
-#if HAVE_LIBAVCODEC
-  if ( vid_stream ) {
-    delete vid_stream;
-    vid_stream = nullptr;
-  }
-#endif
-  delete temp_img_buffer;
+  delete vid_stream;
+  delete[] temp_img_buffer;
   closeComms();
+
+  if (mJpegCodecContext) {
+    avcodec_free_context(&mJpegCodecContext);
+  }
+
+  if (mJpegSwsContext) {
+    sws_freeContext(mJpegSwsContext);
+  }
+}
+
+bool StreamBase::initContexts(int in_width, int in_height, AVPixelFormat format,
+    int out_width, int out_height, unsigned int quality) {
+
+  if (mJpegCodecContext) avcodec_free_context(&mJpegCodecContext);
+
+  std::list<const CodecData *>codec_data = get_encoder_data("mjpeg", "");
+  if (!codec_data.size()) {
+    Error("No codecs for mjpeg found");
+    return false;
+  }
+  for (auto it = codec_data.begin(); it != codec_data.end(); it ++) {
+    auto chosen_codec_data = *it;
+    Debug(1, "Found video codec for %s", chosen_codec_data->codec_name);
+
+    const AVCodec *mJpegCodec = avcodec_find_encoder_by_name(chosen_codec_data->codec_name);
+    if (!mJpegCodec) {
+      Error("MJPEG codec not found");
+      continue;
+    }
+    // We allocate and copy in newer ffmpeg, so need to free it
+    mJpegCodecContext = avcodec_alloc_context3(mJpegCodec);
+    if (!mJpegCodecContext) {
+      Error("Could not allocate jpeg codec context");
+      continue;
+    }
+
+    //mJpegCodecContext->bit_rate = 2000000;
+    mJpegCodecContext->width = out_width;
+    mJpegCodecContext->height = out_height;
+    mJpegCodecContext->time_base= (AVRational) {1, 25};
+    //mJpegCodecContext->time_base= (AVRational) {1, static_cast<int>(monitor->GetFPS())};
+    mJpegCodecContext->pix_fmt = chosen_codec_data->sw_pix_fmt;
+    mJpegCodecContext->sw_pix_fmt = chosen_codec_data->sw_pix_fmt;
+
+    int quality = config.jpeg_file_quality;
+
+    //(alarm_frame && (config.jpeg_alarm_file_quality > config.jpeg_file_quality)) ?
+    //config.jpeg_alarm_file_quality : 0;   // quality to use, zero is default
+    mJpegCodecContext->qcompress = quality/100.0; // 0-1
+    //mJpegCodecContext->qmax = 1;
+    //mJpegCodecContext->qmin = 1; //quality/100.0; // 0-1
+    mJpegCodecContext->global_quality = quality/100.0; // 0-1
+
+    Debug(1, "Setting pix fmt to %d %s, sw_pix_fmt %d %s %dx%d",
+        chosen_codec_data->sw_pix_fmt, av_get_pix_fmt_name(chosen_codec_data->sw_pix_fmt),
+        chosen_codec_data->sw_pix_fmt, av_get_pix_fmt_name(chosen_codec_data->sw_pix_fmt),
+        mJpegCodecContext->width, mJpegCodecContext->height
+        );
+
+#if 0
+    if (0 && setup_hwaccel(mJpegCodecContext,
+          chosen_codec_data, hw_device_ctx, monitor->EncoderHWAccelDevice(), monitor->Width(), monitor->Height())) {
+      avcodec_free_context(&mJpegCodecContext);
+      continue;
+    }
+#endif
+
+    if (avcodec_open2(mJpegCodecContext, mJpegCodec, nullptr) < 0) {
+      Error("Could not open mjpeg codec");
+      avcodec_free_context(&mJpegCodecContext);
+      mJpegCodecContext = nullptr;
+      //av_buffer_unref(&hw_device_ctx);
+      continue;
+    }
+    break;
+  }
+  if (!mJpegCodecContext) {
+    return false;
+  }
+
+  mJpegPixelFormat = format;
+  if (mJpegSwsContext) {
+    //&& (mJpegCodecContext->sw_pix_fmt != format)) {
+    sws_freeContext(mJpegSwsContext);
+  }
+
+  mJpegSwsContext = sws_getContext(
+      //monitor->Width(), monitor->Height(), 
+      // theoretically, the stream can be any size not necessarily monitor size. I think. This is here more for format conversion than scaling.
+      // No we are doing scaling here too. It got removed from prepareImage
+      in_width, in_height, format,
+      out_width, out_height, mJpegCodecContext->pix_fmt,
+      SWS_BICUBIC, nullptr, nullptr, nullptr);
+
+  if (!mJpegSwsContext) {
+    Warning("Failed to alloc swscontext");
+    return false;
+  } else {
+    Debug(1, "Configured swsContext to %dx%d %d %s to %dx%d %d %s",
+        in_width, in_height, format, av_get_pix_fmt_name(format),
+        out_width, out_height, mJpegCodecContext->pix_fmt, av_get_pix_fmt_name(mJpegCodecContext->pix_fmt));
+  }
+  return true;
 }
 
 bool StreamBase::loadMonitor(int p_monitor_id) {
@@ -47,17 +149,16 @@ bool StreamBase::loadMonitor(int p_monitor_id) {
     return false;
   }
 
-  if ( monitor->GetFunction() == Monitor::NONE ) {
-    Info("Monitor %d has function NONE. Will not be able to connect to it.", monitor_id);
-    return false;
+  if (monitor->Capturing() == Monitor::CAPTURING_NONE) {
+    Info("Monitor %d has capturing == NONE. Will likely not be able to connect to it.", monitor_id);
   }
 
-  if ( monitor->isConnected() ) {
+  if (monitor->isConnected()) {
     monitor->disconnect();
   }
 
-  if ( !monitor->connect() ) {
-    Error("Unable to connect to monitor id %d for streaming", monitor_id);
+  if (!monitor->connect()) {
+    Info("Unable to connect to monitor id %d for streaming", monitor_id);
     monitor->disconnect();
     // If we couldn't connect, it might be due to size mismatch in shm. Need to reload
     if ( !(monitor = Monitor::Load(monitor_id, false, Monitor::QUERY)))
@@ -65,6 +166,9 @@ bool StreamBase::loadMonitor(int p_monitor_id) {
     return false;
   }
 
+  mJpegCodecContext = nullptr;
+  mJpegSwsContext = nullptr;
+  //return initContexts(monitor->Width(), monitor->Height(), AV_PIX_FMT_YUV420P, config.jpeg_stream_quality);
   return true;
 }
 
@@ -73,15 +177,14 @@ bool StreamBase::checkInitialised() {
     Error("Cannot stream, not initialised");
     return false;
   }
-  if (monitor->GetFunction() == Monitor::NONE) {
-    Info("Monitor %d has function NONE. Will not be able to connect to it.", monitor_id);
-    return false;
+  if (monitor->Capturing() == Monitor::CAPTURING_NONE) {
+    Info("Monitor %d has capturing == NONE. Will likely not be able to connect to it.", monitor_id);
   }
   if (!monitor->ShmValid()) {
-    Error("Monitor shm is not connected");
+    Debug(1, "Monitor shm is not valid");
     return false;
   }
-  if ((monitor->GetType() == Monitor::FFMPEG) and !monitor->DecodingEnabled() ) {
+  if ((monitor->GetType() == Monitor::FFMPEG) and (monitor->Decoding() == Monitor::DECODING_NONE) ) {
     Debug(1, "Monitor is not decoding.");
     return false;
   }
@@ -89,69 +192,71 @@ bool StreamBase::checkInitialised() {
 }
 
 void StreamBase::updateFrameRate(double fps) {
-  frame_mod = 1;
   if ( (fps < 0) || !fps || std::isinf(fps) ) {
     Debug(1, "Zero or negative fps %f in updateFrameRate. Setting frame_mod=1 and effective_fps=0.0", fps);
     effective_fps = 0.0;
     base_fps = 0.0;
     return;
   }
+
   base_fps = fps;
   effective_fps = (base_fps*abs(replay_rate))/ZM_RATE_BASE;
   frame_mod = 1;
   Debug(3, "FPS:%.2f, MaxFPS:%.2f, BaseFPS:%.2f, EffectiveFPS:%.2f, FrameMod:%d, replay_rate(%d)",
-      fps, maxfps, base_fps, effective_fps, frame_mod, replay_rate);
-  // Min frame repeat?
-  // We want to keep the frame skip easy... problem is ... if effective = 31 and max = 30 then we end up with 15.5 fps.  
-  while ( (int)effective_fps > (int)maxfps ) {
-    effective_fps /= 2.0;
-    frame_mod *= 2;
-    Debug(3, "Changing fps to be < max %.2f EffectiveFPS:%.2f, FrameMod:%d",
-        maxfps, effective_fps, frame_mod);
+        fps, maxfps, base_fps, effective_fps, frame_mod, replay_rate);
+  if (maxfps > 0.0) {
+    // Min frame repeat?
+    // We want to keep the frame skip easy... problem is ... if effective = 31 and max = 30 then we end up with 15.5 fps.
+    while ( (int)effective_fps > (int)maxfps ) {
+      effective_fps /= 2.0;
+      frame_mod *= 2;
+      Debug(3, "Changing fps to be < max %.2f EffectiveFPS:%.2f, FrameMod:%d",
+            maxfps, effective_fps, frame_mod);
+    }
   }
 } // void StreamBase::updateFrameRate(double fps)
 
-bool StreamBase::checkCommandQueue() {
-  if ( sd >= 0 ) {
-    CmdMsg msg;
-    memset(&msg, 0, sizeof(msg));
-    int nbytes = recvfrom(sd, &msg, sizeof(msg), MSG_DONTWAIT, 0, 0);
-    if ( nbytes < 0 ) {
-      if ( errno != EAGAIN ) {
-        Error("recvfrom(), errno = %d, error = %s", errno, strerror(errno));
-        return false;
+void StreamBase::checkCommandQueue() {
+  while (!zm_terminate) {
+    // Update modified time of the socket .lock file so that we can tell which ones are stale.
+    if (now - last_comm_update > Hours(1)) {
+      touch(sock_path_lock);
+      last_comm_update = now;
+    }
+
+    if (sd >= 0) {
+      CmdMsg msg;
+      memset(&msg, 0, sizeof(msg));
+      int nbytes = recvfrom(sd, &msg, sizeof(msg), 0, /*MSG_DONTWAIT*/ 0, 0);
+      if (nbytes < 0) {
+        if (errno != EAGAIN) {
+          Error("recvfrom(), errno = %d, error = %s", errno, strerror(errno));
+        }
+      } else {
+        Debug(2, "Message length is (%d)", nbytes);
+        processCommand(&msg);
+        got_command = true;
       }
+    } else if (connkey) {
+      Warning("No sd in checkCommandQueue, comms not open for connkey %06d?", connkey);
+    } else {
+      // Perfectly valid if only getting a snapshot
+      Debug(1, "No sd in checkCommandQueue, comms not open.");
     }
-    //else if ( (nbytes != sizeof(msg)) )
-    //{
-      //Error( "Partial message received, expected %d bytes, got %d", sizeof(msg), nbytes );
-    //}
-    else {
-      Debug(2, "Message length is (%d)", nbytes);
-      processCommand(&msg);
-      return true;
-    }
-  } else if ( connkey ) {
-    Warning("No sd in checkCommandQueue, comms not open?");
-  } else {
-    // Perfectly valid if only getting a snapshot
-    Debug(1, "No sd in checkCommandQueue, comms not open?");
-  }
-  return false;
-}  // end bool StreamBase::checkCommandQueue()
+  } // end while !zm_terminate
+}  // end void StreamBase::checkCommandQueue()
 
 Image *StreamBase::prepareImage(Image *image) {
   /* zooming should happen before scaling to preserve quality
    * scale is relative to base dimensions, and represents the rough ratio between desired view size and base dimensions
    */
   bool image_copied = false;
+  Debug(1, "prepareImage image in %s", image->toString().c_str());
 
   if (zoom != 100) {
     int base_image_width = image->Width(),
-        base_image_height = image->Height(),
-        disp_image_width = image->Width() * scale/ZM_SCALE_BASE,
-        disp_image_height = image->Height() * scale / ZM_SCALE_BASE;
-    /* x and y are scaled by web UI to base dimensions units. 
+        base_image_height = image->Height();
+    /* x and y are scaled by web UI to base dimensions units.
      * When zooming, we blow up the image by the amount 150 for first zoom, right? 150%, then cut out a base sized chunk
      * However if we have zoomed before, then we are zooming into the previous cutout
      * The box stored in last_crop should be in base_image units, So we need to turn x,y into percentages, then apply to last_crop
@@ -227,21 +332,12 @@ Image *StreamBase::prepareImage(Image *image) {
       image_copied = true;
     }
     image->Crop(last_crop);
-    image->Scale(disp_image_width, disp_image_height);
-  } else if (scale != ZM_SCALE_BASE) {
-    Debug(3, "scaling by %d from %dx%d", scale, image->Width(), image->Height());
-    static Image copy_image;
-    copy_image.Assign(*image);
-    image = &copy_image;
-    image_copied = true;
-    image->Scale(scale);
   }
-  Debug(3, "Sending %dx%d", image->Width(), image->Height());
-
   last_scale = scale;
   last_zoom = zoom;
   last_x = x;
   last_y = y;
+  Debug(3, "prepareImage image in %s", image->toString().c_str());
 
   return image;
 }  // end Image *StreamBase::prepareImage(Image *image)
@@ -249,19 +345,19 @@ Image *StreamBase::prepareImage(Image *image) {
 bool StreamBase::sendTextFrame(const char *frame_text) {
   int width = 640;
   int height = 480;
-  int colours = ZM_COLOUR_RGB32;
-  int subpixelorder = ZM_SUBPIX_ORDER_RGBA;
+  int colours = ZM_COLOUR_YUV420P;
+  int subpixelorder = ZM_SUBPIX_ORDER_YUV420P;
   int labelsize = 2;
 
   if (monitor) {
     width = monitor->Width();
     height = monitor->Height();
-    colours = monitor->Colours();
-    subpixelorder = monitor->SubpixelOrder();
+    //colours = monitor->Colours();
+    //subpixelorder = monitor->SubpixelOrder();
     labelsize = monitor->LabelSize();
   }
-  Debug(2, "Sending %dx%d * %d text frame '%s'",
-      width, height, scale, frame_text);
+  Debug(2, "Sending %dx%d colours:%d subpixel:%d * %d scale text frame '%s'",
+        width, height, colours, subpixelorder, scale, frame_text);
 
   Image image(width, height, colours, subpixelorder);
   image.Clear();
@@ -269,21 +365,27 @@ bool StreamBase::sendTextFrame(const char *frame_text) {
 
   if (scale != 100) {
     image.Scale(scale);
+    Debug(2, "Scaled by %d to %dx%d size %d", scale, image.Width(), image.Height(), image.Size());
   }
-#if HAVE_LIBAVCODEC
-  if ( type == STREAM_MPEG ) {
-    if ( !vid_stream ) {
+  if (type == STREAM_MPEG) {
+    if (!vid_stream) {
       vid_stream = new VideoStream("pipe:", format, bitrate, effective_fps, image.Colours(), image.SubpixelOrder(), image.Width(), image.Height());
       fprintf(stdout, "Content-Type: %s\r\n\r\n", vid_stream->MimeType());
       vid_stream->OpenStream();
     }
     /* double pts = */ vid_stream->EncodeFrame(image.Buffer(), image.Size());
-  } else
-#endif // HAVE_LIBAVCODEC
-  {
+  } else {
     static unsigned char buffer[ZM_MAX_IMAGE_SIZE];
-    int n_bytes = 0;
+    size_t n_bytes = 0;
 
+    AVPixelFormat pixformat = image.AVPixFormat();
+    if ((!mJpegCodecContext) ||
+          mJpegCodecContext->width != width 
+        || mJpegCodecContext->height != height 
+        || mJpegPixelFormat != pixformat) {
+      Debug(1, "Need to reinit contexts");
+      initContexts(width, height, pixformat, width, height, config.jpeg_stream_quality);
+    }
     image.EncodeJpeg(buffer, &n_bytes);
 
     if (type == STREAM_JPEG) {
@@ -292,19 +394,20 @@ bool StreamBase::sendTextFrame(const char *frame_text) {
         return false;
       }
     }
-    if (0 > fprintf(stdout, "Content-Type: image/jpeg\r\nContent-Length: %d\r\n\r\n", n_bytes)) {
-      Debug(1, "Error sending Content-Type: image/jpeg\r\nContent-Length: %d\r\n\r\n", n_bytes);
+    if (0 > fprintf(stdout, "Content-Type: image/jpeg\r\nContent-Length: %zu\r\n\r\n", n_bytes)) {
+      Debug(1, "Error sending Content-Type: image/jpeg\r\nContent-Length: %zu\r\n\r\n", n_bytes);
       return false;
     }
     int rc = fwrite(buffer, n_bytes, 1, stdout);
     if (rc != 1) {
-      Error("Unable to send stream text frame: %d %s", rc, strerror(errno));
+      if (!zm_terminate)
+        Error("Unable to send stream text frame: %d %s", rc, strerror(errno));
       return false;
     }
     fputs("\r\n\r\n", stdout);
     fflush(stdout);
   }
-  last_frame_sent = TV_2_FLOAT(now);
+  last_frame_sent = now;
   return true;
 }
 
@@ -319,12 +422,12 @@ void StreamBase::openComms() {
     }
 
     unsigned int length = snprintf(
-        sock_path_lock,
-        sizeof(sock_path_lock),
-        "%s/zms-%06d.lock",
-        staticConfig.PATH_SOCKS.c_str(),
-        connkey
-        );
+                            sock_path_lock,
+                            sizeof(sock_path_lock),
+                            "%s/zms-%06d.lock",
+                            staticConfig.PATH_SOCKS.c_str(),
+                            connkey
+                          );
     if ( length >= sizeof(sock_path_lock) ) {
       Warning("Socket lock path was truncated.");
     }
@@ -362,12 +465,12 @@ void StreamBase::openComms() {
     }
 
     length = snprintf(
-        loc_sock_path,
-        sizeof(loc_sock_path),
-        "%s/zms-%06ds.sock",
-        staticConfig.PATH_SOCKS.c_str(),
-        connkey
-        );
+               loc_sock_path,
+               sizeof(loc_sock_path),
+               "%s/zms-%06ds.sock",
+               staticConfig.PATH_SOCKS.c_str(),
+               connkey
+             );
     if ( length >= sizeof(loc_sock_path) ) {
       Warning("Socket path was truncated.");
       length = sizeof(loc_sock_path)-1;
@@ -389,9 +492,12 @@ void StreamBase::openComms() {
     strncpy(rem_addr.sun_path, rem_sock_path, sizeof(rem_addr.sun_path));
     rem_addr.sun_family = AF_UNIX;
 
-    gettimeofday(&last_comm_update, nullptr);
+    struct timeval tv {1,0}; /* 1 Secs Timeout */
+    setsockopt(sd, SOL_SOCKET, SO_RCVTIMEO,(struct timeval *)&tv, sizeof(struct timeval));
+
+    last_comm_update = std::chrono::steady_clock::now();
+    Debug(3, "comms open at %s", loc_sock_path);
   } // end if connKey > 0
-  Debug(3, "comms open at %s", loc_sock_path);
 } // end void StreamBase::openComms()
 
 void StreamBase::closeComms() {
@@ -402,7 +508,16 @@ void StreamBase::closeComms() {
     }
     // Can't delete any files because another zms might have come along and opened them and is waiting on the lock.
     if ( lock_fd > 0 ) {
-      close(lock_fd); //close it rather than unlock it incase it got deleted.
+      close(lock_fd); //close it rather than unlock it in case it got deleted.
     }
   }
 } // end void StreamBase::closeComms
+
+void StreamBase::reserveTempImgBuffer(size_t size) {
+  if (temp_img_buffer_size < size) {
+    Debug(1, "Resizing image buffer from %zu to %zu", temp_img_buffer_size, size);
+    delete[] temp_img_buffer;
+    temp_img_buffer = new uint8_t[size];
+    temp_img_buffer_size = size;
+  }
+}
