@@ -24,6 +24,8 @@
 #include "zm_monitorstream.h"
 #include "zm_eventstream.h"
 #include "zm_fifo_stream.h"
+#include <iomanip>
+#include <sstream>
 #include <string>
 #include <unistd.h>
 
@@ -34,16 +36,16 @@ bool ValidateAccess(User *user, int mon_id) {
     if (user->getStream() < User::PERM_VIEW) {
       allowed = false;
       Warning("Insufficient privileges for request user %d %s for monitor %d, user does not have permission to stream live view.",
-          user->Id(), user->getUsername(), mon_id);
+              user->Id(), user->getUsername(), mon_id);
     } else if (!user->canAccess(mon_id)) {
       allowed = false;
       Warning("Insufficient privileges for request user %d %s for monitor %d, user does not have permission view this monitor.",
-          user->Id(), user->getUsername(), mon_id);
+              user->Id(), user->getUsername(), mon_id);
     }
   } else if (user->getEvents() < User::PERM_VIEW) {
     allowed = false;
     Warning("Insufficient privileges for request user %d %s, user does not have permission view events.",
-        user->Id(), user->getUsername());
+            user->Id(), user->getUsername());
   }
   return allowed;
 }
@@ -69,11 +71,13 @@ int main(int argc, const char *argv[], char **envp) {
   time_t event_time = 0;
   uint64_t event_id = 0;
   unsigned int frame_id = 1;
+  int frames_to_send = -1;
   unsigned int scale = 100;
   unsigned int rate = 100;
-  double maxfps = 10.0;
+  double maxfps = 0.0;
   unsigned int bitrate = 100000;
   unsigned int ttl = 0;
+  bool  analysis_frames = false;
   EventStream::StreamMode replay = EventStream::MODE_NONE;
   std::string username;
   std::string password;
@@ -126,7 +130,14 @@ int main(int argc, const char *argv[], char **envp) {
     char const *value = strtok(nullptr, "=");
     if ( !value )
       value = "";
-    if ( !strcmp(name, "source") ) {
+    if ( !strcmp(name, "analysis") ) {
+      if ( !strcmp(value, "true") ) {
+        analysis_frames = true;
+      } else {
+        analysis_frames = (atoi(value) == 1);
+      }
+      Debug(1, "Viewing analysis frames");
+    } else if ( !strcmp(name, "source") ) {
       if ( !strcmp(value, "event") ) {
         source = ZMS_EVENT;
       } else if ( !strcmp(value, "fifo") ) {
@@ -145,6 +156,12 @@ int main(int argc, const char *argv[], char **envp) {
       monitor_id = atoi(value);
       if ( source == ZMS_UNKNOWN )
         source = ZMS_MONITOR;
+    } else if ( !strcmp(name, "datetime") ) {
+      std::tm tm = {};
+      std::stringstream ss(value);
+      ss >> std::get_time(&tm, "%Y-%m-%d %H:%M:%S");
+      auto tp = std::chrono::system_clock::from_time_t(std::mktime(&tm));
+      event_time = std::chrono::duration_cast<FPSeconds>(tp.time_since_epoch()).count();
     } else if ( !strcmp(name, "time") ) {
       event_time = atoi(value);
     } else if ( !strcmp(name, "event") ) {
@@ -153,6 +170,8 @@ int main(int argc, const char *argv[], char **envp) {
     } else if ( !strcmp(name, "frame") ) {
       frame_id = strtoull(value, nullptr, 10);
       source = ZMS_EVENT;
+    } else if ( !strcmp(name, "frames") ) {
+      frames_to_send = strtoll(value, nullptr, 10);
     } else if ( !strcmp(name, "scale") ) {
       scale = atoi(value);
       if (scale > 1600) {
@@ -190,18 +209,23 @@ int main(int argc, const char *argv[], char **envp) {
       Debug(1, "ZMS: JWT token found: %s", jwt_token_str.c_str());
     } else if ( !strcmp(name, "user") ) {
       username = UriDecode(value);
+    } else if ( !strcmp(name, "username") ) {
+      username = UriDecode(value);
     } else if ( !strcmp(name, "pass") ) {
       password = UriDecode(value);
       Debug(1, "Have %s for password", password.c_str());
+    } else if ( !strcmp(name, "password") ) {
+      password = UriDecode(value);
+      Debug(1, "Have %s for password", password.c_str());
     } else {
       Debug(1, "Unknown parameter passed to zms %s=%s", name, value);
     }  // end if possible parameter names
   }  // end foreach parm
 
-  if ( monitor_id ) {
-    snprintf(log_id_string, sizeof(log_id_string), "zms_m%d", monitor_id);
-  } else {
+  if ( event_id ) {
     snprintf(log_id_string, sizeof(log_id_string), "zms_e%" PRIu64, event_id);
+  } else if ( monitor_id ) {
+    snprintf(log_id_string, sizeof(log_id_string), "zms_m%d", monitor_id);
   }
   logInit(log_id_string);
 
@@ -219,17 +243,13 @@ int main(int argc, const char *argv[], char **envp) {
       }
     } else {
       if ( *auth ) {
-        user = zmLoadAuthUser(auth, config.auth_hash_ips);
+        user = zmLoadAuthUser(auth, username, config.auth_hash_ips);
       } else if ( username.length() && password.length() ) {
         user = zmLoadUser(username.c_str(), password.c_str());
       }
     }
     if ( !user ) {
-      if (nph) {
-        fputs("HTTP/1.0 403 Forbidden\r\n\r\n", stdout);
-      } else {
-        fputs("Status: 403\r\nContent-Type: text/html\r\n", stdout);
-      }
+      fputs("HTTP/1.0 403 Forbidden\r\n\r\n", stdout);
 
       const char *referer = getenv("HTTP_REFERER");
       Error("Unable to authenticate user from %s", referer);
@@ -238,11 +258,7 @@ int main(int argc, const char *argv[], char **envp) {
     if ( !ValidateAccess(user, monitor_id) ) {
       delete user;
       user = nullptr;
-      if (nph) {
-        fputs("HTTP/1.0 403 Forbidden\r\n\r\n", stdout);
-      } else {
-        fputs("Status: 403\r\nContent-Type: text/html\r\n\r\n", stdout);
-      }
+      fputs("HTTP/1.0 403 Forbidden\r\n\r\n", stdout);
       return exit_zms(0);
     }
     delete user;
@@ -260,20 +276,20 @@ int main(int argc, const char *argv[], char **envp) {
   }
   fprintf(stdout, "Server: ZoneMinder Video Server/%s\r\n", ZM_VERSION);
 
-  time_t now = time(nullptr);
+  time_t now = std::chrono::system_clock::to_time_t(std::chrono::system_clock::now());
   char date_string[64];
   tm now_tm = {};
   strftime(date_string, sizeof(date_string)-1,
-      "%a, %d %b %Y %H:%M:%S GMT", gmtime_r(&now, &now_tm));
+           "%a, %d %b %Y %H:%M:%S GMT", gmtime_r(&now, &now_tm));
 
   fputs("Last-Modified: ", stdout);
   fputs(date_string, stdout);
   fputs(
-      "\r\nExpires: Mon, 26 Jul 1997 05:00:00 GMT\r\n"
-      "Cache-Control: no-store, no-cache, must-revalidate\r\n"
-      "Cache-Control: post-check=0, pre-check=0\r\n"
-      "Pragma: no-cache\r\n",
-      stdout);
+    "\r\nExpires: Mon, 26 Jul 1997 05:00:00 GMT\r\n"
+    "Cache-Control: no-store, no-cache, must-revalidate\r\n"
+    "Cache-Control: post-check=0, pre-check=0\r\n"
+    "Pragma: no-cache\r\n",
+    stdout);
 
   if ( source == ZMS_MONITOR ) {
     MonitorStream stream;
@@ -283,7 +299,9 @@ int main(int argc, const char *argv[], char **envp) {
     stream.setStreamTTL(ttl);
     stream.setStreamQueue(connkey);
     stream.setStreamBuffer(playback_buffer);
+    stream.setFramesToSend(frames_to_send);
     stream.setStreamStart(monitor_id);
+    stream.setStreamFrameType(analysis_frames ? StreamBase::FRAME_ANALYSIS: StreamBase::FRAME_NORMAL);
 
     if ( mode == ZMS_JPEG ) {
       stream.setStreamType(MonitorStream::STREAM_JPEG);
@@ -294,19 +312,9 @@ int main(int argc, const char *argv[], char **envp) {
     } else if ( mode == ZMS_SINGLE ) {
       stream.setStreamType(MonitorStream::STREAM_SINGLE);
     } else {
-#if HAVE_LIBAVCODEC
       stream.setStreamFormat(format);
       stream.setStreamBitrate(bitrate);
       stream.setStreamType(MonitorStream::STREAM_MPEG);
-#else  // HAVE_LIBAVCODEC
-      Error("MPEG streaming of '%s' attempted while disabled", query);
-      fprintf(stderr, "MPEG streaming is disabled.\n"
-          "You should configure with the --with-ffmpeg"
-          " option and rebuild to use this functionality.\n");
-      logTerm();
-      zmDbClose();
-      return -1;
-#endif  // HAVE_LIBAVCODEC
     }
     stream.runStream();
   } else if ( source == ZMS_FIFO ) {
@@ -324,28 +332,22 @@ int main(int argc, const char *argv[], char **envp) {
     stream.setStreamMaxFPS(maxfps);
     stream.setStreamMode(replay);
     stream.setStreamQueue(connkey);
+    stream.setFramesToSend(frames_to_send);
     if ( monitor_id && event_time ) {
       stream.setStreamStart(monitor_id, event_time);
     } else {
       Debug(3, "Setting stream start to frame (%d)", frame_id);
       stream.setStreamStart(event_id, frame_id);
     }
+    stream.setStreamFrameType(analysis_frames ? StreamBase::FRAME_ANALYSIS: StreamBase::FRAME_NORMAL);
     if ( mode == ZMS_JPEG ) {
       stream.setStreamType(EventStream::STREAM_JPEG);
+    } else if ( mode == ZMS_SINGLE ) {
+      stream.setStreamType(MonitorStream::STREAM_SINGLE);
     } else {
-#if HAVE_LIBAVCODEC
       stream.setStreamFormat(format);
       stream.setStreamBitrate(bitrate);
       stream.setStreamType(EventStream::STREAM_MPEG);
-#else  // HAVE_LIBAVCODEC
-      Error("MPEG streaming of '%s' attempted while disabled", query);
-      fprintf(stderr, "MPEG streaming is disabled.\n"
-          "You should ensure the ffmpeg libraries are installed and detected"
-          " and rebuild to use this functionality.\n");
-      logTerm();
-      zmDbClose();
-      return -1;
-#endif  // HAVE_LIBAVCODEC
     }  // end if jpeg or mpeg
     stream.runStream();
   } else {
@@ -354,3 +356,4 @@ int main(int argc, const char *argv[], char **envp) {
 
   return exit_zms(0);
 }
+
