@@ -1,21 +1,21 @@
 //
 // ZoneMinder Control Utility, $Date$, $Revision$
 // Copyright (C) 2001-2008 Philip Coombes
-// 
+//
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
 // as published by the Free Software Foundation; either version 2
 // of the License, or (at your option) any later version.
-// 
+//
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
-// 
+//
 
 /*
 
@@ -36,15 +36,15 @@ zmc - The ZoneMinder Utility
 This binary is a handy command line interface to several useful functions. It's
 not really meant to be used by anyone except the web page (there's only limited
 'help' in it so far) but can be if necessary, especially for debugging video
-problems. 
+problems.
 
 =head1 OPTIONS
 
 General options:
   -v, --verbose                           - Produce more verbose output
   -l, --list                              - List the current status of active (or all with -v) monitors
-  -h, --help                               - Display usage information
-  -v, --version                            - Print the installed version of ZoneMinder
+  -h, --help                              - Display usage information
+  -v, --version                           - Print the installed version of ZoneMinder
 
 Options for use with devices:
   -d, --device [device_path]              - Get the current video device settings for [device_path] or all devices
@@ -54,12 +54,12 @@ Options for use with devices:
 Options for use with monitors:
   -m, --monitor <monitor_id>              - Specify which monitor to address, default 1 if absent
   -q, --query                             - Query the current settings for the monitor
-  -s, --state                             - Output the current monitor state, 0 = idle, 1 = prealarm, 2 = alarm,
-                                            3 = alert, 4 = tape
-  -B, --brightness [value]                - Output the current brightness, set to value if given 
-  -C, --contrast [value]                  - Output the current contrast, set to value if given 
-  -H, --hue [value]                       - Output the current hue, set to value if given 
-  -O, --colour [value]                    - Output the current colour, set to value if given 
+  -s, --state                             - Output the current monitor state, 0 = unknown, 1 = idle, 2 = prealarm,
+                                            3 = alarm, 4 = alert, 5 = tape
+  -B, --brightness [value]                - Output the current brightness, set to value if given
+  -C, --contrast [value]                  - Output the current contrast, set to value if given
+  -H, --hue [value]                       - Output the current hue, set to value if given
+  -O, --colour [value]                    - Output the current colour, set to value if given
   -i, --image [image_index]               - Write captured image to disk as <monitor_name>.jpg, last image captured
                                             or specified ring buffer index if given.
   -S, --scale <scale_%%ge>                - With --image specify any scaling (in %%) to be applied to the image
@@ -81,7 +81,7 @@ Options for use with monitors:
   -U, --username <username>               - When running in authenticated mode the username and
   -P, --password <password>               - password combination of the given user
   -A, --auth <authentication>             - Pass authentication hash string instead of user details
-  -x, --xtrigger                          - Output the current monitor trigger state, 0 = not triggered, 1 = triggered 
+  -x, --xtrigger                          - Output the current monitor trigger state, 0 = not triggered, 1 = triggered
 =cut
 
 */
@@ -97,84 +97,86 @@ Options for use with monitors:
 
 void Usage(int status=-1) {
   fputs(
-			"zmu <-d device_path> [-v] [function] [-U<username> -P<password>]\n"
-			"zmu <-m monitor_id> [-v] [function] [-U<username> -P<password>]\n"
-			"General options:\n"
-			"  -h, --help           : This screen\n"
-			"  -v, --verbose          : Produce more verbose output\n"
-			"  -l, --list           : List the current status of active (or all with -v) monitors\n"
-			"Options for use with devices:\n"
-			"  -d, --device [device_path]   : Get the current video device settings for [device_path] or all devices\n"
-			"  -V, --version <V4L version>  : Set the Video 4 Linux API version to use for the query, use 1 or 2\n"
-			"  -q, --query          : Query the current settings for the device\n"
-			"Options for use with monitors:\n"
-			"  -m, --monitor <monitor_id>   : Specify which monitor to address, default 1 if absent\n"
-			"  -q, --query          : Query the current settings for the monitor\n"
-			"  -s, --state          : Output the current monitor state, 0 = idle, 1 = prealarm, 2 = alarm,\n"
-			"                   3 = alert, 4 = tape\n"
-			"  -B, --brightness [value]     : Output the current brightness, set to value if given \n"
-			"  -C, --contrast [value]     : Output the current contrast, set to value if given \n"
-			"  -H, --hue [value]        : Output the current hue, set to value if given \n"
-			"  -O, --colour [value]       : Output the current colour, set to value if given \n"
-			"  -i, --image [image_index]    : Write captured image to disk as <monitor_name>.jpg, last image captured\n"
-			"                   or specified ring buffer index if given.\n"
-			"  -S, --scale <scale_%%ge>    : With --image specify any scaling (in %%) to be applied to the image\n"
-			"  -t, --timestamp [image_index]  : Output captured image timestamp, last image captured or specified\n"
-			"                   ring buffer index if given\n"
-			"  -R, --read_index         : Output ring buffer read index\n"
-			"  -W, --write_index        : Output ring buffer write index\n" 
-			"  -e, --event          : Output last event index\n" 
-			"  -f, --fps            : Output last Frames Per Second captured reading\n" 
-			"  -z, --zones          : Write last captured image overlaid with zones to <monitor_name>-Zones.jpg\n" 
-			"  -a, --alarm          : Force alarm in monitor, this will trigger recording until cancelled with -c\n" 
-			"  -n, --noalarm          : Force no alarms in monitor, this will prevent alarms until cancelled with -c\n" 
-			"  -c, --cancel           : Cancel a forced alarm/noalarm in monitor, required after being enabled with -a or -n\n" 
-			"  -L, --reload           : Signal monitor to reload settings\n" 
-			"  -E, --enable           : Enable detection, wake monitor up\n" 
-			"  -D, --disable          : Disable detection, put monitor to sleep\n" 
-			"  -u, --suspend          : Suspend detection, useful to prevent bogus alarms when panning etc\n" 
-			"  -r, --resume           : Resume detection after a suspend\n" 
-			"  -U, --username <username>    : When running in authenticated mode the username and\n" 
-			"  -P, --password <password>    : password combination of the given user\n" 
-			"  -A, --auth <authentication>  : Pass authentication hash string instead of user details\n"
-                        "  -T, --token <token>  : Pass JWT token string instead of user details\n"
-                        "  -x, --xtrigger       : Output the current monitor trigger state, 0 = not triggered, 1 = triggered\n"
+    "zmu <-d device_path> [-v] [function] [-U<username> -P<password>]\n"
+    "zmu <-m monitor_id> [-v] [function] [-U<username> -P<password>]\n"
+    "General options:\n"
+    "  -h, --help           : This screen\n"
+    "  -v, --verbose          : Produce more verbose output\n"
+    "  -l, --list           : List the current status of active (or all with -v) monitors\n"
+    "Options for use with devices:\n"
+    "  -d, --device [device_path]   : Get the current video device settings for [device_path] or all devices\n"
+    "  -V, --version <V4L version>  : Set the Video 4 Linux API version to use for the query, use 1 or 2\n"
+    "  -q, --query          : Query the current settings for the device\n"
+    "Options for use with monitors:\n"
+    "  -m, --monitor <monitor_id>   : Specify which monitor to address, default 1 if absent\n"
+    "  -q, --query          : Query the current settings for the monitor\n"
+    "  -s, --state          : Output the current monitor state, 0 = idle, 1 = prealarm, 2 = alarm,\n"
+    "                   3 = alert, 4 = tape\n"
+    "  -j, --janus-pin     : Output the pin, if set, used to secure Janus for this monitor \n"
+    "  -B, --brightness [value]     : Output the current brightness, set to value if given \n"
+    "  -C, --contrast [value]     : Output the current contrast, set to value if given \n"
+    "  -H, --hue [value]        : Output the current hue, set to value if given \n"
+    "  -O, --colour [value]       : Output the current colour, set to value if given \n"
+    "  -i, --image [image_index]    : Write captured image to disk as <monitor_name>.jpg, last image captured\n"
+    "                   or specified ring buffer index if given.\n"
+    "  -S, --scale <scale_%%ge>    : With --image specify any scaling (in %%) to be applied to the image\n"
+    "  -t, --timestamp [image_index]  : Output captured image timestamp, last image captured or specified\n"
+    "                   ring buffer index if given\n"
+    "  -R, --read_index         : Output ring buffer read index\n"
+    "  -W, --write_index        : Output ring buffer write index\n"
+    "  -e, --event          : Output last event index\n"
+    "  -f, --fps            : Output last Frames Per Second captured reading\n"
+    "  -z, --zones          : Write last captured image overlaid with zones to <monitor_name>-Zones.jpg\n"
+    "  -a, --alarm          : Force alarm in monitor, this will trigger recording until cancelled with -c\n"
+    "  -n, --noalarm          : Force no alarms in monitor, this will prevent alarms until cancelled with -c\n"
+    "  -c, --cancel           : Cancel a forced alarm/noalarm in monitor, required after being enabled with -a or -n\n"
+    "  -L, --reload           : Signal monitor to reload settings\n"
+    "  -E, --enable           : Enable capture, wake monitor up\n"
+    "  -D, --disable          : Disable capture, put monitor to sleep\n"
+    "  -u, --suspend          : Suspend motion detection, useful to prevent bogus alarms when panning etc\n"
+    "  -r, --resume           : Resume motion detection after a suspend\n"
+    "  -U, --username <username>    : When running in authenticated mode the username and\n"
+    "  -P, --password <password>    : password combination of the given user\n"
+    "  -A, --auth <authentication>  : Pass authentication hash string instead of user details\n"
+    "  -T, --token <token>  : Pass JWT token string instead of user details\n"
+    "  -x, --xtrigger       : Output the current monitor trigger state, 0 = not triggered, 1 = triggered\n"
 
-	 "", stderr );
+    "", stderr );
 
   exit(status);
 }
 
 typedef enum {
-	ZMU_BOGUS      = 0x00000000,
-	ZMU_STATE      = 0x00000001,
-	ZMU_IMAGE      = 0x00000002,
-	ZMU_TIME       = 0x00000004,
-	ZMU_READ_IDX   = 0x00000008,
-	ZMU_WRITE_IDX  = 0x00000010,
-	ZMU_EVENT      = 0x00000020,
-	ZMU_FPS        = 0x00000040,
-	ZMU_ZONES      = 0x00000080,
-	ZMU_ALARM      = 0x00000100,
-	ZMU_NOALARM    = 0x00000200,
-	ZMU_CANCEL     = 0x00000400,
-	ZMU_QUERY      = 0x00000800,
-	ZMU_BRIGHTNESS = 0x00001000,
-	ZMU_CONTRAST   = 0x00002000,
-	ZMU_HUE        = 0x00004000,
-	ZMU_COLOUR     = 0x00008000,
-	ZMU_RELOAD     = 0x00010000,
-	ZMU_ENABLE     = 0x00100000,
-	ZMU_DISABLE    = 0x00200000,
-	ZMU_SUSPEND    = 0x00400000,
-	ZMU_RESUME     = 0x00800000,
-	ZMU_LIST       = 0x10000000,
-        ZMU_TRIGGER    = 0x20000000,
+  ZMU_BOGUS      = 0x00000000,
+  ZMU_STATE      = 0x00000001,
+  ZMU_IMAGE      = 0x00000002,
+  ZMU_TIME       = 0x00000004,
+  ZMU_READ_IDX   = 0x00000008,
+  ZMU_WRITE_IDX  = 0x00000010,
+  ZMU_EVENT      = 0x00000020,
+  ZMU_FPS        = 0x00000040,
+  ZMU_ZONES      = 0x00000080,
+  ZMU_ALARM      = 0x00000100,
+  ZMU_NOALARM    = 0x00000200,
+  ZMU_CANCEL     = 0x00000400,
+  ZMU_QUERY      = 0x00000800,
+  ZMU_BRIGHTNESS = 0x00001000,
+  ZMU_CONTRAST   = 0x00002000,
+  ZMU_HUE        = 0x00004000,
+  ZMU_COLOUR     = 0x00008000,
+  ZMU_RELOAD     = 0x00010000,
+  ZMU_ENABLE     = 0x00100000,
+  ZMU_DISABLE    = 0x00200000,
+  ZMU_SUSPEND    = 0x00400000,
+  ZMU_RESUME     = 0x00800000,
+  ZMU_LIST       = 0x10000000,
+  ZMU_TRIGGER    = 0x20000000,
+  ZMU_JANUS      = 0x40000000,
 } Function;
 
 bool ValidateAccess(User *user, int mon_id, int function) {
   bool allowed = true;
-  if ( function & (ZMU_STATE|ZMU_IMAGE|ZMU_TIME|ZMU_READ_IDX|ZMU_WRITE_IDX|ZMU_FPS|ZMU_TRIGGER) ) {
+  if ( function & (ZMU_STATE|ZMU_IMAGE|ZMU_TIME|ZMU_READ_IDX|ZMU_WRITE_IDX|ZMU_FPS|ZMU_TRIGGER|ZMU_JANUS) ) {
     if ( user->getStream() < User::PERM_VIEW )
       allowed = false;
   }
@@ -224,6 +226,7 @@ int main(int argc, char *argv[]) {
     {"scale", 1, nullptr, 'S'},
     {"timestamp", 2, nullptr, 't'},
     {"state", 0, nullptr, 's'},
+    {"janus-pin", 0, nullptr, 'j'},
     {"brightness", 2, nullptr, 'B'},
     {"contrast", 2, nullptr, 'C'},
     {"hue", 2, nullptr, 'H'},
@@ -272,154 +275,153 @@ int main(int argc, char *argv[]) {
   bool have_colour = false;
 
   char *zoneString = nullptr;
-  char *username = nullptr;
-  char *password = nullptr;
+  std::string username;
+  std::string password;
   char *auth = nullptr;
   std::string jwt_token_str = "";
-#if ZM_HAS_V4L
 #if ZM_HAS_V4L2
-    int v4lVersion = 2;
-#elif ZM_HAS_V4L1
-    int v4lVersion = 1;
-#endif // ZM_HAS_V4L2/1
-#endif // ZM_HAS_V4L
+  int v4lVersion = 2;
+#endif // ZM_HAS_V4L2
   while (1) {
     int option_index = 0;
 
-    int c = getopt_long(argc, argv, "d:m:vsEDLurweix::S:t::fz::ancqhlB::C::H::O::RWU:P:A:V:T:", long_options, &option_index);
+    int c = getopt_long(argc, argv, "d:m:vsjEDLurweix::S:t::fz::ancqhlB::C::H::O::RWU:P:A:V:T:", long_options, &option_index);
     if (c == -1) {
       break;
     }
 
     switch (c) {
-      case 'd':
-        if (optarg)
-          device = optarg;
-        break;
-      case 'm':
-        mon_id = atoi(optarg);
-        break;
-      case 'v':
-        verbose = true;
-        break;
-      case 's':
-        function |= ZMU_STATE;
-        break;
-      case 'x':
-        function |= ZMU_TRIGGER;
-        break;
-      case 'i':
-        function |= ZMU_IMAGE;
-        if (optarg)
-          image_idx = atoi(optarg);
-        break;
-      case 'S':
-        scale = atoi(optarg);
-        break;
-      case 't':
-        function |= ZMU_TIME;
-        if (optarg)
-          image_idx = atoi(optarg);
-        break;
-      case 'R':
-        function |= ZMU_READ_IDX;
-        break;
-      case 'W':
-        function |= ZMU_WRITE_IDX;
-        break;
-      case 'e':
-        function |= ZMU_EVENT;
-        break;
-      case 'f':
-        function |= ZMU_FPS;
-        break;
-      case 'z':
-        function |= ZMU_ZONES;
-        if (optarg)
-          zoneString = optarg;
-        break;
-      case 'a':
-        function |= ZMU_ALARM;
-        break;
-      case 'n':
-        function |= ZMU_NOALARM;
-        break;
-      case 'c':
-        function |= ZMU_CANCEL;
-        break;
-      case 'L':
-        function |= ZMU_RELOAD;
-        break;
-      case 'E':
-        function |= ZMU_ENABLE;
-        break;
-      case 'D':
-        function |= ZMU_DISABLE;
-        break;
-      case 'u':
-        function |= ZMU_SUSPEND;
-        break;
-      case 'r':
-        function |= ZMU_RESUME;
-        break;
-      case 'q':
-        function |= ZMU_QUERY;
-        break;
-      case 'B':
-        function |= ZMU_BRIGHTNESS;
-        if (optarg) {
-          have_brightness = true;
-          brightness = atoi(optarg);
-        }
-        break;
-      case 'C':
-        function |= ZMU_CONTRAST;
-        if (optarg) {
-          have_contrast = true;
-          contrast = atoi(optarg);
-        }
-        break;
-      case 'H':
-        function |= ZMU_HUE;
-        if (optarg) {
-          have_hue = true;
-          hue = atoi(optarg);
-        }
-        break;
-      case 'O':
-        function |= ZMU_COLOUR;
-        if (optarg) {
-          have_colour = true;
-          colour = atoi(optarg);
-        }
-        break;
-      case 'U':
-        username = optarg;
-        break;
-      case 'P':
-        password = optarg;
-        break;
-      case 'A':
-        auth = optarg;
-        break;
-      case 'T':
-        jwt_token_str = std::string(optarg);
-        break;
-#if ZM_HAS_V4L
-			case 'V':
-				v4lVersion = (atoi(optarg)==1)?1:2;
-				break;
-#endif // ZM_HAS_V4L
-      case 'h':
-      case '?':
-        Usage(0);
-        break;
-      case 'l':
-        function |= ZMU_LIST;
-        break;
-      default:
-        //fprintf( stderr, "?? getopt returned character code 0%o ??\n", c );
-        break;
+    case 'd':
+      if (optarg)
+        device = optarg;
+      break;
+    case 'm':
+      mon_id = atoi(optarg);
+      break;
+    case 'v':
+      verbose = true;
+      break;
+    case 's':
+      function |= ZMU_STATE;
+      break;
+    case 'j':
+      function |= ZMU_JANUS;
+      break;
+    case 'x':
+      function |= ZMU_TRIGGER;
+      break;
+    case 'i':
+      function |= ZMU_IMAGE;
+      if (optarg)
+        image_idx = atoi(optarg);
+      break;
+    case 'S':
+      scale = atoi(optarg);
+      break;
+    case 't':
+      function |= ZMU_TIME;
+      if (optarg)
+        image_idx = atoi(optarg);
+      break;
+    case 'R':
+      function |= ZMU_READ_IDX;
+      break;
+    case 'W':
+      function |= ZMU_WRITE_IDX;
+      break;
+    case 'e':
+      function |= ZMU_EVENT;
+      break;
+    case 'f':
+      function |= ZMU_FPS;
+      break;
+    case 'z':
+      function |= ZMU_ZONES;
+      if (optarg)
+        zoneString = optarg;
+      break;
+    case 'a':
+      function |= ZMU_ALARM;
+      break;
+    case 'n':
+      function |= ZMU_NOALARM;
+      break;
+    case 'c':
+      function |= ZMU_CANCEL;
+      break;
+    case 'L':
+      function |= ZMU_RELOAD;
+      break;
+    case 'E':
+      function |= ZMU_ENABLE;
+      break;
+    case 'D':
+      function |= ZMU_DISABLE;
+      break;
+    case 'u':
+      function |= ZMU_SUSPEND;
+      break;
+    case 'r':
+      function |= ZMU_RESUME;
+      break;
+    case 'q':
+      function |= ZMU_QUERY;
+      break;
+    case 'B':
+      function |= ZMU_BRIGHTNESS;
+      if (optarg) {
+        have_brightness = true;
+        brightness = atoi(optarg);
+      }
+      break;
+    case 'C':
+      function |= ZMU_CONTRAST;
+      if (optarg) {
+        have_contrast = true;
+        contrast = atoi(optarg);
+      }
+      break;
+    case 'H':
+      function |= ZMU_HUE;
+      if (optarg) {
+        have_hue = true;
+        hue = atoi(optarg);
+      }
+      break;
+    case 'O':
+      function |= ZMU_COLOUR;
+      if (optarg) {
+        have_colour = true;
+        colour = atoi(optarg);
+      }
+      break;
+    case 'U':
+      username = optarg;
+      break;
+    case 'P':
+      password = optarg;
+      break;
+    case 'A':
+      auth = optarg;
+      break;
+    case 'T':
+      jwt_token_str = std::string(optarg);
+      break;
+#if ZM_HAS_V4L2
+    case 'V':
+      v4lVersion = (atoi(optarg)==1)?1:2;
+      break;
+#endif // ZM_HAS_V4L2
+    case 'h':
+    case '?':
+      Usage(0);
+      break;
+    case 'l':
+      function |= ZMU_LIST;
+      break;
+    default:
+      //fprintf( stderr, "?? getopt returned character code 0%o ??\n", c );
+      break;
     }
   } // end getopt loop
 
@@ -456,27 +458,27 @@ int main(int argc, char *argv[]) {
     if ( jwt_token_str != "" ) {
       user = zmLoadTokenUser(jwt_token_str, false);
     } else if ( strcmp(config.auth_relay, "none") == 0 ) {
-      if ( !username ) {
+      if (username.empty()) {
         Error("Username must be supplied");
         exit_zmu(-1);
       }
 
-      if ( !checkUser(username)) {
+      if (!checkUser(username)) {
         Error("Username greater than allowed 32 characters");
         exit_zmu(-1);
       }
 
       user = zmLoadUser(username);
     } else {
-       
-      if ( !(username && password) && !auth ) {
+
+      if ( !(!username.empty() && !password.empty()) && !auth ) {
         Error("Username and password or auth/token string must be supplied");
         exit_zmu(-1);
       }
       if ( auth ) {
-        user = zmLoadAuthUser(auth, false);
+        user = zmLoadAuthUser(auth, username, false);
       }
-      if ( username && password ) {
+      if ( !username.empty() && !password.empty() ) {
         if ( !checkUser(username)) {
           Error("username greater than allowed 32 characters");
           exit_zmu(-1);
@@ -492,10 +494,10 @@ int main(int argc, char *argv[]) {
       Error("Unable to authenticate user");
       exit_zmu(-1);
     }
-		if ( !ValidateAccess(user, mon_id, function) ) {
-			Error("Insufficient privileges for user %s for requested function %x", username, function);
-			exit_zmu(-1);
-		}
+    if ( !ValidateAccess(user, mon_id, function) ) {
+      Error("Insufficient privileges for user %s for requested function %x", username.c_str(), function);
+      exit_zmu(-1);
+    }
   } // end if auth
 
   if ( mon_id > 0 ) {
@@ -508,6 +510,16 @@ int main(int argc, char *argv[]) {
     if ( verbose ) {
       printf("Monitor %u(%s)\n", monitor->Id(), monitor->Name());
     }
+
+    if (monitor->Capturing() == Monitor::CAPTURING_NONE) {
+      if (verbose) {
+        printf("Current capturing state: None\n");
+      } else {
+        printf("%d", Monitor::UNKNOWN);
+      }
+      exit_zmu(-1);
+    }
+
     if ( !monitor->connect() ) {
       Error("Can't connect to capture daemon: %d %s", monitor->Id(), monitor->Name());
       exit_zmu(-1);
@@ -515,6 +527,10 @@ int main(int argc, char *argv[]) {
 
     char separator = ' ';
     bool have_output = false;
+    if ( function & ZMU_JANUS ) {
+      printf("%s", monitor->get_stream_key());
+      have_output = true;
+    }
     if ( function & ZMU_STATE ) {
       Monitor::State state = monitor->GetState();
       if ( verbose ) {
@@ -534,20 +550,26 @@ int main(int argc, char *argv[]) {
       }
     }
     if ( function & ZMU_TIME ) {
-      struct timeval timestamp = monitor->GetTimestamp(image_idx);
-      if ( verbose ) {
+      SystemTimePoint timestamp = monitor->GetTimestamp(image_idx);
+      if (verbose) {
         char timestamp_str[64] = "None";
-        if ( timestamp.tv_sec ) {
+        if (timestamp.time_since_epoch() != Seconds(0)) {
           tm tm_info = {};
-          strftime(timestamp_str, sizeof(timestamp_str), "%Y-%m-%d %H:%M:%S", localtime_r(&timestamp.tv_sec, &tm_info));
+          time_t timestamp_t = std::chrono::system_clock::to_time_t(timestamp);
+          strftime(timestamp_str, sizeof(timestamp_str), "%Y-%m-%d %H:%M:%S", localtime_r(&timestamp_t, &tm_info));
+        }
+        Seconds ts_sec = std::chrono::duration_cast<Seconds>(timestamp.time_since_epoch());
+        Microseconds ts_usec = std::chrono::duration_cast<Microseconds>(timestamp.time_since_epoch() - ts_sec);
+        if (image_idx == -1) {
+          printf("Time of last image capture: %s.%02d\n", timestamp_str, static_cast<int32>(ts_usec.count()));
+        } else {
+          printf("Time of image %d capture: %s.%02d\n", image_idx, timestamp_str, static_cast<int32>(ts_usec.count()));
         }
-        if ( image_idx == -1 )
-          printf("Time of last image capture: %s.%02ld\n", timestamp_str, timestamp.tv_usec/10000);
-        else
-          printf("Time of image %d capture: %s.%02ld\n", image_idx, timestamp_str, timestamp.tv_usec/10000);
       } else {
-        if ( have_output ) fputc(separator, stdout);
-        printf("%ld.%02ld", timestamp.tv_sec, timestamp.tv_usec/10000);
+        if (have_output) {
+          fputc(separator, stdout);
+        }
+        printf("%.2f", FPSeconds(timestamp.time_since_epoch()).count());
         have_output = true;
       }
     }
@@ -581,7 +603,7 @@ int main(int argc, char *argv[]) {
     if ( function & ZMU_FPS ) {
       if ( verbose ) {
         printf("Current capture rate: %.2f frames per second, analysis rate: %.2f frames per second\n",
-            monitor->get_capture_fps(), monitor->get_analysis_fps());
+               monitor->get_capture_fps(), monitor->get_analysis_fps());
       } else {
         if ( have_output ) fputc(separator, stdout);
         printf("capture: %.2f, analysis: %.2f", monitor->get_capture_fps(), monitor->get_analysis_fps());
@@ -606,32 +628,31 @@ int main(int argc, char *argv[]) {
       monitor->DumpZoneImage(zoneString);
     }
     if ( function & ZMU_ALARM ) {
-      if ( monitor->GetFunction() == Monitor::Function::MONITOR ) {
-        printf("A Monitor in monitor mode cannot handle alarms.  Please use NoDect\n");
-      } else {
-        Monitor::State state = monitor->GetState();
+      Monitor::State state = monitor->GetState();
 
-        if ( verbose ) {
-          printf("Forcing alarm on current state: %s, event %" PRIu64 "\n",
-              state==Monitor::ALARM?"Alarm":(state==Monitor::ALERT?"Alert":"Idle"),
-              monitor->GetLastEventId()
+      if (verbose) {
+        printf("Forcing alarm on current state: %s, event %" PRIu64 "\n",
+               state==Monitor::ALARM?"Alarm":(state==Monitor::ALERT?"Alert":"Idle"),
+               monitor->GetLastEventId()
               );
-        }
+      }
 
-        // Ensure that we are not recording.  So the forced alarm is distinct from what was recording before
-        monitor->ForceAlarmOff();
-        monitor->ForceAlarmOn(config.forced_alarm_score, "Forced Web");
-        int wait = 10*1000*1000; // 10 seconds
-        while ((monitor->GetState() != Monitor::ALARM) and !zm_terminate and wait) {
-          // Wait for monitor to notice.
-          usleep(1000);
-          wait -= 1000;
-        }
-        if ( monitor->GetState() != Monitor::ALARM and !wait ) {
-          Error("Monitor failed to respond to forced alarm.");
-        } else {
-          printf("Alarmed event id: %" PRIu64 "\n", monitor->GetLastEventId());
-        }
+      // Ensure that we are not recording.  So the forced alarm is distinct from what was recording before
+      monitor->ForceAlarmOff();
+      monitor->ForceAlarmOn(config.forced_alarm_score, "Forced Web");
+
+      Microseconds wait_time = Seconds(10);
+      while ((monitor->GetState() != Monitor::ALARM) and !zm_terminate and wait_time > Seconds(0)) {
+        // Wait for monitor to notice.
+        Microseconds sleep = Microseconds(1);
+        std::this_thread::sleep_for(sleep);
+        wait_time -= sleep;
+      }
+
+      if (monitor->GetState() != Monitor::ALARM and wait_time == Seconds(0)) {
+        Error("Monitor failed to respond to forced alarm.");
+      } else {
+        printf("Alarmed event id: %" PRIu64 "\n", monitor->GetLastEventId());
       }
     }  // end if ZMU_ALARM
 
@@ -652,22 +673,22 @@ int main(int argc, char *argv[]) {
     }
     if ( function & ZMU_ENABLE ) {
       if ( verbose )
-        printf("Enabling event generation\n");
+        printf("Enabling capturing\n");
       monitor->actionEnable();
     }
     if ( function & ZMU_DISABLE ) {
       if ( verbose )
-        printf("Disabling event generation\n");
+        printf("Disabling capturing\n");
       monitor->actionDisable();
     }
     if ( function & ZMU_SUSPEND ) {
       if ( verbose )
-        printf("Suspending event generation\n");
+        printf("Suspending motion detection\n");
       monitor->actionSuspend();
     }
     if ( function & ZMU_RESUME ) {
       if ( verbose )
-        printf("Resuming event generation\n");
+        printf("Resuming motion detection\n");
       monitor->actionResume();
     }
     if ( function & ZMU_QUERY ) {
@@ -744,70 +765,75 @@ int main(int argc, char *argv[]) {
     }
   } else { // non monitor functions
     if ( function & ZMU_QUERY ) {
-#if ZM_HAS_V4L
-			char vidString[0x10000] = "";
-			bool ok = LocalCamera::GetCurrentSettings(device.empty()?nullptr:device.c_str(), vidString, v4lVersion, verbose);
-			printf("%s", vidString);
-			exit_zmu(ok ? 0 : -1);
-#else // ZM_HAS_V4L
-			Error("Video4linux is required for device querying");
+#if ZM_HAS_V4L2
+      char vidString[0x10000] = "";
+      bool ok = LocalCamera::GetCurrentSettings(device, vidString, v4lVersion, verbose);
+      printf("%s", vidString);
+      exit_zmu(ok ? 0 : -1);
+#else // ZM_HAS_V4L2
+      Error("Video4linux is required for device querying");
       exit_zmu(-1);
-#endif // ZM_HAS_V4L
+#endif // ZM_HAS_V4L2
     }
-
-    if ( function & ZMU_LIST ) {
-      std::string sql = "SELECT `Id`, `Function`+0 FROM `Monitors`";
-      if (!verbose) {
-        sql += "WHERE `Function` != 'None'";
-      }
-      sql += " ORDER BY Id ASC";
-
-      MYSQL_RES *result = zmDbFetch(sql.c_str());
-      if (!result) {
-        exit_zmu(-1);
-      }
-      Debug(1, "Got %" PRIu64 " monitors", static_cast<uint64>(mysql_num_rows(result)));
-
-      printf("%4s%5s%6s%9s%14s%6s%6s%8s%8s\n", "Id", "Func", "State", "TrgState", "LastImgTim", "RdIdx", "WrIdx", "LastEvt", "FrmRate");
-      for ( int i = 0; MYSQL_ROW dbrow = mysql_fetch_row(result); i++ ) {
-        int monitor_id = atoi(dbrow[0]);
-        int monitor_function = atoi(dbrow[1]);
-        if ( !user || user->canAccess(monitor_id) ) {
-          if ( monitor_function > 1 ) {
-            std::shared_ptr<Monitor> monitor = Monitor::Load(monitor_id, false, Monitor::QUERY);
-            if ( monitor && monitor->connect() ) {
-              struct timeval tv = monitor->GetTimestamp();
-              printf( "%4d%5d%6d%9d%11ld.%02ld%6d%6d%8" PRIu64 "%8.2f\n",
-                monitor->Id(),
-                monitor_function,
-                monitor->GetState(),
-                monitor->GetTriggerState(),
-                tv.tv_sec, tv.tv_usec/10000,
-                monitor->GetLastReadIndex(),
-                monitor->GetLastWriteIndex(),
-                monitor->GetLastEventId(),
-                monitor->GetFPS()
-              );
-            }
-          } else {
-            struct timeval tv = { 0, 0 };
-            printf("%4d%5d%6d%9d%11ld.%02ld%6d%6d%8d%8.2f\n",
-              mon_id,
-              function,
-              0,
-              0,
-              tv.tv_sec, tv.tv_usec/10000,
-              0,
-              0,
-              0,
-              0.0
-            );
-          } // end if function filter
-        } // endif !user || canAccess(mon_id)
-      } // end foreach row
-      mysql_free_result(result);
-    } // end if function && ZMU_LIST
   } // end if monitor id or not
+
+  if (function & ZMU_LIST) {
+    std::string sql = "SELECT `Id`, `Capturing`+0, `Analysing`+0, `Recording`+0 FROM `Monitors`";
+    if (!verbose) {
+      sql += " WHERE `Capturing` != 'None'";
+    }
+    sql += " ORDER BY Id ASC";
+
+    MYSQL_RES *result = zmDbFetch(sql);
+    if (!result) {
+      exit_zmu(-1);
+    }
+    Debug(1, "Got %" PRIu64 " monitors", static_cast<uint64>(mysql_num_rows(result)));
+
+    printf("%4s %9s %9s %9s %5s %8s %13s %5s %5s %9s %9s\n",
+           "Id", "Capturing", "Analysing", "Recording", "State", "TrgState",
+           "LastImageTime", "RdIdx", "WrIdx", "LastEvent", "FrameRate");
+    for (int i=0; MYSQL_ROW dbrow = mysql_fetch_row(result); i++) {
+      int monitor_id = atoi(dbrow[0]);
+      if (mon_id and (monitor_id != mon_id)) continue;
+      if (!user || user->canAccess(monitor_id)) {
+        int monitor_capturing = atoi(dbrow[1]);
+        if (monitor_capturing > Monitor::CAPTURING_NONE) {
+          std::shared_ptr<Monitor> monitor = Monitor::Load(monitor_id, false, Monitor::QUERY);
+          if (monitor && monitor->connect()) {
+            SystemTimePoint timestamp = monitor->GetTimestamp();
+
+            printf("%4d %9d %9d %9d %5d %8d %13.2f %5d %5d %9" PRIu64 "%10.2f\n",
+                   monitor->Id(),
+                   monitor->Capturing(),
+                   monitor->Analysing(),
+                   monitor->Recording(),
+                   monitor->GetState(),
+                   monitor->GetTriggerState(),
+                   FPSeconds(timestamp.time_since_epoch()).count(),
+                   monitor->GetLastReadIndex(),
+                   monitor->GetLastWriteIndex(),
+                   monitor->GetLastEventId(),
+                   monitor->GetFPS()
+                  );
+          }
+        } else {
+          printf("%4d%5d%6d%9d%11ld.%02ld%6d%6d%8d%8.2f\n",
+                 monitor_id,
+                 function,
+                 0,
+                 0,
+                 0l, 0l,
+                 0,
+                 0,
+                 0,
+                 0.0
+                );
+        } // end if function filter
+      } // endif !user || canAccess(mon_id)
+    } // end foreach row
+    mysql_free_result(result);
+  } // end if function && ZMU_LIST
   delete user;
 
   exit_zmu(0);
