@@ -22,7 +22,7 @@ class EventsController extends AppController {
   public function beforeFilter() {
     parent::beforeFilter();
     global $user;
-    $canView = (!$user) || ($user['Events'] != 'None');
+    $canView = (!$user) || ($user->Events() != 'None');
     if ( !$canView ) {
       throw new UnauthorizedException(__('Insufficient Privileges'));
       return;
@@ -40,16 +40,16 @@ class EventsController extends AppController {
 
     global $user;
     require_once __DIR__ .'/../../../includes/Event.php';
-    $allowedMonitors = $user ? preg_split('@,@', $user['MonitorIds'], NULL, PREG_SPLIT_NO_EMPTY) : null;
+    $allowedMonitors = ($user and $user->unviewableMonitorIds()) ? $user->viewableMonitorIds() : [];
 
-    if ( $allowedMonitors ) {
+    if (count($allowedMonitors)) {
       $mon_options = array('Event.MonitorId' => $allowedMonitors);
     } else {
       $mon_options = '';
     }
 
     $named_params = $this->request->params['named'];
-    if ( $named_params ) {
+    if ($named_params) {
       # In 1.35.13 we renamed StartTime and EndTime to StartDateTime and EndDateTime.
       # This hack renames the query string params
       foreach ( $named_params as $k=>$v ) {
@@ -78,11 +78,12 @@ class EventsController extends AppController {
       // with many event APIs. In future, we can
       // make a nice ZM_API_ITEMS_PER_PAGE for all pagination
       // API
+      // ICON: 2023-11-16: MontageReview now uses API and unless we specifiy a limit in params, there should be no limit
+      // TODO: Implement request based limits.
 
-      'limit' => '100',
-      'order' => array('StartDateTime'),
       'paramType' => 'querystring',
     );
+
     if ( isset($conditions['GroupId']) ) {
       $settings['joins'] = array(
         array(
@@ -97,22 +98,19 @@ class EventsController extends AppController {
     }
     $settings['conditions'] = array($conditions, $mon_options);
 
-    // How many events to return 
-    $this->loadModel('Config');
-    $limit = $this->Config->find('list', array(
-      'conditions' => array('Name' => 'ZM_WEB_EVENTS_PER_PAGE'),
-      'fields' => array('Name', 'Value')
-    ));
     $this->Paginator->settings = $settings;
-    $events = $this->Paginator->paginate('Event');
-
-    // For each event, get the frameID which has the largest score
-    // also add FS path
+    if ($this->request->query('limit') or $this->request->query('page')) {
+      $events = $this->Paginator->paginate('Event');
+    } else {
+      $events = $this->Event->find('all', $settings);
+    }
+    // For each event, get the frameID which has the largest score also add FS path
 
     foreach ( $events as $key => $value ) {
       $EventObj = new ZM\Event($value['Event']);
-      $maxScoreFrameId = $this->getMaxScoreAlarmFrameId($value['Event']['Id']);
-      $events[$key]['Event']['MaxScoreFrameId'] = $maxScoreFrameId;
+      if ($EventObj->MaxScoreFrameId() == NULL) {
+        $events[$key]['Event']['MaxScoreFrameId'] = $this->getMaxScoreAlarmFrameId($value['Event']['Id']);
+      }
       $events[$key]['Event']['FileSystemPath'] = $EventObj->Path();
     }
 
@@ -135,7 +133,7 @@ class EventsController extends AppController {
     }
 
     global $user;
-    $allowedMonitors = $user ? preg_split('@,@', $user['MonitorIds'], NULL, PREG_SPLIT_NO_EMPTY) : null;
+    $allowedMonitors = ($user and $user->unviewableMonitorIds()) ? $user->viewableMonitorIds() : null;
 
     if ( $allowedMonitors ) {
       $mon_options = array('Event.MonitorId' => $allowedMonitors);
@@ -153,8 +151,8 @@ class EventsController extends AppController {
     # Get the previous and next events for any monitor
     $this->Event->id = $id;
     $event_neighbors = $this->Event->find('neighbors');
-    $event['Event']['Next'] = $event_neighbors['next']['Event']['Id'];
-    $event['Event']['Prev'] = $event_neighbors['prev']['Event']['Id'];
+    $event['Event']['Next'] = isset($event_neighbors['next']) ? $event_neighbors['next']['Event']['Id'] : 0;
+    $event['Event']['Prev'] = isset($event_neighbors['prev']) ? $event_neighbors['prev']['Event']['Id'] : 0;
 
     $event['Event']['fileExists'] = $this->Event->fileExists($event['Event']);
     $event['Event']['fileSize'] = $this->Event->fileSize($event['Event']);
@@ -166,14 +164,15 @@ class EventsController extends AppController {
     $event_monitor_neighbors = $this->Event->find('neighbors', array(
       'conditions'=>array('Event.MonitorId'=>$event['Event']['MonitorId'])
     ));
-    $event['Event']['NextOfMonitor'] = $event_monitor_neighbors['next']['Event']['Id'];
-    $event['Event']['PrevOfMonitor'] = $event_monitor_neighbors['prev']['Event']['Id'];
+    $event['Event']['NextOfMonitor'] = isset($event_monitor_neighbors['next']) ? $event_monitor_neighbors['next']['Event']['Id'] : 0;
+    $event['Event']['PrevOfMonitor'] = isset($event_monitor_neighbors['prev']) ? $event_monitor_neighbors['prev']['Event']['Id'] : 0;
   
     $this->loadModel('Frame');
     $maxScoreFrame = $this->Frame->findByEventid($id, 'FrameId', array('Score'=>'desc','FrameId'=>'asc'));
 
     $event['Event']['MaxScoreFrameId'] = $maxScoreFrame ? $maxScoreFrame['Frame']['FrameId'] : null;
     $alarmFrame = $this->Frame->findByEventidAndType($id, 'Alarm');
+    ZM\Debug(print_r($alarmFrame, true));
 
     $event['Event']['AlarmFrameId'] = $alarmFrame ? $alarmFrame['Frame']['FrameId'] : null;
 
@@ -192,7 +191,7 @@ class EventsController extends AppController {
   public function add() {
 
     global $user;
-    $canEdit = (!$user) || ($user['Events'] == 'Edit');
+    $canEdit = (!$user) || ($user->Events() == 'Edit');
     if ( !$canEdit ) {
       throw new UnauthorizedException(__('Insufficient privileges'));
       return;
@@ -218,7 +217,7 @@ class EventsController extends AppController {
   public function edit($id = null) {
 
     global $user;
-    $canEdit = (!$user) || ($user['Events'] == 'Edit');
+    $canEdit = (!$user) || ($user->Events() == 'Edit');
     if ( !$canEdit ) {
       throw new UnauthorizedException(__('Insufficient privileges'));
       return;
@@ -251,7 +250,7 @@ class EventsController extends AppController {
    */
   public function delete($id = null) {
     global $user;
-    $canEdit = (!$user) || ($user['Events'] == 'Edit');
+    $canEdit = (!$user) || ($user->Events() == 'Edit');
     if ( !$canEdit ) {
       throw new UnauthorizedException(__('Insufficient privileges'));
       return;
@@ -334,12 +333,9 @@ class EventsController extends AppController {
     } 
     array_push($conditions, array("StartDateTime >= DATE_SUB(NOW(), INTERVAL $expr $unit)"));
     $query = $this->Event->find('all', array(
-                                             'fields' => array(
-                                                               'MonitorId',
-                                                               'COUNT(*) AS Count',
-                                                               ),
-                                             'conditions' => $conditions,
-                                             'group' => 'MonitorId',
+      'fields' => array('MonitorId', 'COUNT(*) AS Count'),
+      'conditions' => $conditions,
+      'group' => 'MonitorId',
     ));
 
     foreach ($query as $result) {
