@@ -148,12 +148,30 @@ class Monitor extends AppModel {
       'joinTable' =>  'Event_Summaries',
     )
   );
+  
+  public $belongsTo = array(
+    'Manufacturer' => array(
+      'className' => 'Manufacturer',
+      'foreignKey' => 'ManufacturerId',
+      'joinTable' => 'Manufacturers',
+    ),
+    'CameraModel' => array(
+      'className' => 'CameraModel',
+      'foreignKey' => 'ModelId',
+      'joinTable' => 'Models',
+    )
+  );
 
   public function daemonControl($monitor, $command, $daemon=null) {
-    if ($monitor['Function'] == 'None' and $command != 'stop') {
+    ZM\Warning(print_r($monitor, true));
+    if (isset($monitor['Function']) and ($monitor['Function'] == 'None') and $command != 'stop') {
       ZM\Warning("Calling daemonControl with command $command when Function == None");
       return;
     }
+    if ($monitor['Capturing'] == 'None' and $command != 'stop') {
+      ZM\Warning("Calling daemonControl with command $command when Capturing == None");
+      return;
+    }
     if (defined('ZM_SERVER_ID') and ($monitor['ServerId']!=ZM_SERVER_ID)) {
       ZM\Error('Calling daemonControl for Monitor assigned to different server. Our server id '.ZM_SERVER_ID.' != '.$monitor['ServerId']);
       return;
