@@ -18,6 +18,18 @@
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 //
 
+$Go2RTCTypes = array(
+  'WebRTC' => 'WebRTC',
+  'MSE' => 'MSE',
+  #'RTSP' => 'RTSP',
+  'HLS' => 'HLS',
+  'MJPEG' => 'MJPEG'
+);
+$RTSP2WebTypes = array(
+  'HLS' => 'HLS',
+  'MSE' => 'MSE',
+  'WebRTC' => 'WebRTC',
+);
 $rates = array(
   -1600 => '-16x',
   -1000 => '-10x',
@@ -37,23 +49,45 @@ $rates = array(
 );
 
 $scales = array(
-    '0' => translate('Auto'),
-    '400' => '4x',
-    '300' => '3x',
-    '200' => '2x',
-    '150' => '1.5x',
-    '100' => translate('Actual'),
-    '75' => '3/4x',
-    '50' => '1/2x',
-    '33' => '1/3x',
-    '25' => '1/4x',
-    '12.5' => '1/8x',
+  # We use 0 instead of words because we are saving this in the monitor
+  # and use this array to populate the default scale option
+  '0' => translate('Auto'),
+  #  '400' => '4x',
+  #  '300' => '3x',
+  #  '200' => '2x',
+  #  '150' => '1.5x',
+  '100' => translate('Actual'),
+  #  '75' => '3/4x',
+  #  '50' => '1/2x',
+  #  '33' => '1/3x',
+  #  '25' => '1/4x',
+  #  '12.5' => '1/8x',
+  'fit_to_width' => translate('Fit to width'),
+  '480px' => translate('Max 480px'),
+  '640px' => translate('Max 640px'),
+  '800px' => translate('Max 800px'),
+  '1024px' => translate('Max 1024px'),
+  '1280px' => translate('Max 1280px'),
+  '1600px' => translate('Max 1600px'),
+);
+
+$streamQuality = array(
+  # In %
+  '+50' => '+50%',
+  '+40' => '+40%',
+  '+30' => '+30%',
+  '+20' => '+20%',
+  '+10' => '+10%',
+  '0' => translate('Optimal'),
+  '-10' => '-10%',
+  '-20' => '-20%',
+  '-30' => '-30%',
+  '-40' => '-40%',
+  '-50' => '-50%',
 );
 
 if ( isset($_REQUEST['view']) && ($_REQUEST['view'] == 'montage') ) {
   unset($scales['auto']); //Remove auto on montage, use everywhere else
-} else {
-  unset($scales['fixed']); //Remove fixed on everything but montage
 }
 
 $bandwidth_options = array(
@@ -80,6 +114,7 @@ switch ( $_COOKIE['zmBandwidth'] ) {
         define( 'ZM_WEB_EVENTS_VIEW', ZM_WEB_H_EVENTS_VIEW );           // What the default view of multiple events should be.
         define( 'ZM_WEB_SHOW_PROGRESS', ZM_WEB_H_SHOW_PROGRESS );       // Whether to show the progress of replay in event view.
         define( 'ZM_WEB_AJAX_TIMEOUT', ZM_WEB_H_AJAX_TIMEOUT );         // Timeout to use for Ajax requests, no timeout used if unset
+        define( 'ZM_WEB_VIEWING_TIMEOUT', defined('ZM_WEB_H_VIEWING_TIMEOUT') ? ZM_WEB_H_VIEWING_TIMEOUT : 0 );
         break;
     } case 'medium' : {
         define( 'ZM_WEB_REFRESH_MAIN', ZM_WEB_M_REFRESH_MAIN );         // How often (in seconds) the main console window refreshes
@@ -98,6 +133,7 @@ switch ( $_COOKIE['zmBandwidth'] ) {
         define( 'ZM_WEB_EVENTS_VIEW', ZM_WEB_M_EVENTS_VIEW );           // What the default view of multiple events should be.
         define( 'ZM_WEB_SHOW_PROGRESS', ZM_WEB_M_SHOW_PROGRESS );       // Whether to show the progress of replay in event view.
         define( 'ZM_WEB_AJAX_TIMEOUT', ZM_WEB_M_AJAX_TIMEOUT );         // Timeout to use for Ajax requests, no timeout used if unset
+        define( 'ZM_WEB_VIEWING_TIMEOUT', defined('ZM_WEB_M_VIEWING_TIMEOUT') ? ZM_WEB_M_VIEWING_TIMEOUT : 0 );
         break;
     } case 'low' : {
         define( 'ZM_WEB_REFRESH_MAIN', ZM_WEB_L_REFRESH_MAIN );         // How often (in seconds) the main console window refreshes
@@ -116,6 +152,7 @@ switch ( $_COOKIE['zmBandwidth'] ) {
         define( 'ZM_WEB_EVENTS_VIEW', ZM_WEB_L_EVENTS_VIEW );           // What the default view of multiple events should be.
         define( 'ZM_WEB_SHOW_PROGRESS', ZM_WEB_L_SHOW_PROGRESS );       // Whether to show the progress of replay in event view.
         define( 'ZM_WEB_AJAX_TIMEOUT', ZM_WEB_L_AJAX_TIMEOUT );         // Timeout to use for Ajax requests, no timeout used if unset
+        define( 'ZM_WEB_VIEWING_TIMEOUT', defined('ZM_WEB_L_VIEWING_TIMEOUT') ? ZM_WEB_L_VIEWING_TIMEOUT : 0 );
         break;
     }
 }
