@@ -1,90 +1,72 @@
-var optControl = <?php echo ZM_OPT_CONTROL ?>;
-var hasOnvif = <?php echo ZM_HAS_ONVIF ?>;
-var defaultAspectRatio = '<?php echo ZM_DEFAULT_ASPECT_RATIO ?>';
+const hasOnvif = <?php echo ZM_HAS_ONVIF ?>;
+const defaultAspectRatio = '<?php echo ZM_DEFAULT_ASPECT_RATIO ?>';
+const messageSavingDataWhenLeavingPage = '<?php echo translate('MessageSavingDataWhenLeavingPage') ?>';
 
 <?php
-if ( ZM_OPT_CONTROL ) {
+if (ZM_OPT_CONTROL and canView('Control')) {
 ?>
-var controlOptions = new Object();
+const controlOptions = new Object();
 <?php
   global $controls;
-  foreach ( $controls as $control ) {
-    echo '
+  if ($controls) {
+    foreach ($controls as $control) {
+      echo '
 controlOptions['.$control->Id().'] = new Array();
 controlOptions['.$control->Id().'][0] = '.
-    ( $control->HasHomePreset() ? '\''.translate('Home').'\'' : 'null' ).PHP_EOL;
-    for ( $i = 1; $i <= $control->NumPresets(); $i++ ) {
-      echo 'controlOptions['. $control->Id().']['.$i.'] = \''.translate('Preset').' '.$i .'\';'.PHP_EOL;
-    }
-  } # end foreach row
+      ( $control->HasHomePreset() ? '\''.translate('Home').'\'' : 'null' ).PHP_EOL;
+      for ( $i = 1; $i <= $control->NumPresets(); $i++ ) {
+        echo 'controlOptions['. $control->Id().']['.$i.'] = \''.translate('Preset').' '.$i .'\';'.PHP_EOL;
+      }
+    } # end foreach row
+  }
 } # end if ZM_OPT_CONTROL
 ?>
 
-var monitorNames = new Object();
-var rtspStreamNames = new Object();
+  const od_models = <?php global $models; echo json_encode($models,JSON_PRETTY_PRINT|JSON_NUMERIC_CHECK); ?>;
+const monitorNames = new Object();
+const rtspStreamNames = new Object();
 <?php
-$query = empty($_REQUEST['mid']) ?
-  dbQuery('SELECT Name,RTSPStreamName FROM Monitors') :
-  dbQuery('SELECT Name,RTSPStreamName FROM Monitors WHERE Id != ?', array($_REQUEST['mid']) );
+$mid = empty($_REQUEST['mid']) ? '0' : validCardinal($_REQUEST['mid']);
+$query = $mid ?
+  dbQuery('SELECT Name,RTSPStreamName FROM Monitors WHERE Id != ? AND Deleted=false', array($mid)):
+  dbQuery('SELECT Name,RTSPStreamName FROM Monitors WHERE Deleted=false');
 if ($query) {
   while ($row = dbFetchNext($query)) {
-    echo '
-monitorNames[\''.validJsStr($row['Name']).'\'] = true;
-rtspStreamNames[\''.validJsStr($row['RTSPStreamName']).'\'] = true;
-';
+    echo 'monitorNames[\''.validJsStr($row['Name']).'\'] = true;'.PHP_EOL;
+    if ($row['RTSPStreamName'])
+      echo 'rtspStreamNames[\''.validJsStr($row['RTSPStreamName']).'\'] = true;'.PHP_EOL;
   } // end foreach
-} # end if query
+} # end if $query
+echo 'const mid='.$mid.';'.PHP_EOL;
 ?>
 
-function validateForm() {
-  const form = document.getElementById('contentForm');
+function validateForm(form) {
   const errors = new Array();
   const warnings = new Array();
   const elements = form.elements;
 
   // No monitor input should have whitespace at beginning or end, so strip them out first.
-  for (var i=0; i<elements.length; i++) {
+  for (let i=0; i<elements.length; i++) {
     if (elements[i].nodeName != 'SELECT') {
       elements[i].value = elements[i].value.trim();
     }
   }
 
-  if ( elements['newMonitor[Name]'].value.search( /[^\w\-\.\(\)\:\/ ]/ ) >= 0 ) {
+  if ( elements['newMonitor[Name]'].value.search( /[^\w\-\.\(\)\:\/ ]/ ) >= 0 )
     errors[errors.length] = "<?php echo translate('BadNameChars') ?>";
-  } else if ( monitorNames[form.elements['newMonitor[Name]'].value] ) {
+  else if ( monitorNames[form.elements['newMonitor[Name]'].value] )
     errors[errors.length] = "<?php echo translate('DuplicateMonitorName') ?>";
-  }
-  if ( form.elements['newMonitor[Type]'].value != 'WebSite' ) {
-    if ( form.elements['newMonitor[AnalysisFPSLimit]'].value && !(parseFloat(form.elements['newMonitor[AnalysisFPSLimit]'].value) > 0 ) )
-      errors[errors.length] = "<?php echo translate('BadAnalysisFPS') ?>";
-    if ( form.elements['newMonitor[MaxFPS]'].value && !(parseFloat(form.elements['newMonitor[MaxFPS]'].value) > 0 ) )
-      errors[errors.length] = "<?php echo translate('BadMaxFPS') ?>";
-    if ( form.elements['newMonitor[AlarmMaxFPS]'].value && !(parseFloat(form.elements['newMonitor[AlarmMaxFPS]'].value) > 0 ) )
-      errors[errors.length] = "<?php echo translate('BadAlarmMaxFPS') ?>";
-    if ( !form.elements['newMonitor[RefBlendPerc]'].value || (parseInt(form.elements['newMonitor[RefBlendPerc]'].value) > 100 ) || (parseInt(form.elements['newMonitor[RefBlendPerc]'].value) < 0 ) )
-      errors[errors.length] = "<?php echo translate('BadRefBlendPerc') ?>";
-  }
-  if (errors.length) {
-    $j('#general-tab').tab('show');
-    alert(errors.join("\n"));
-    return false;
-  }
 
   if ( form.elements['newMonitor[Type]'].value == 'Local' ) {
-    if ( !form.elements['newMonitor[Palette]'].value || !form.elements['newMonitor[Palette]'].value.match( /^\d+$/ ) ) {
+    if ( !form.elements['newMonitor[Palette]'] || !form.elements['newMonitor[Palette]'].value || !form.elements['newMonitor[Palette]'].value.match( /^\d+$/ ) )
       errors[errors.length] = "<?php echo translate('BadPalette') ?>";
-    }
-    if ( !form.elements['newMonitor[Device]'].value ) {
+    if ( !form.elements['newMonitor[Device]'].value )
       errors[errors.length] = "<?php echo translate('BadDevice') ?>";
-    }
-
-    if ( !form.elements['newMonitor[Channel]'].value || !form.elements['newMonitor[Channel]'].value.match( /^\d+$/ ) ) {
+    if ( !form.elements['newMonitor[Channel]'] || !form.elements['newMonitor[Channel]'].value || !form.elements['newMonitor[Channel]'].value.match( /^\d+$/ ) )
       errors[errors.length] = "<?php echo translate('BadChannel') ?>";
-    }
-    if ( !form.elements['newMonitor[Format]'].value || !form.elements['newMonitor[Format]'].value.match( /^\d+$/ ) ) {
+    if ( !form.elements['newMonitor[Format]'] || !form.elements['newMonitor[Format]'].value || !form.elements['newMonitor[Format]'].value.match( /^\d+$/ ) )
       errors[errors.length] = "<?php echo translate('BadFormat') ?>";
-    }
-    if ( form.elements['newMonitor[VideoWriter]'].value == 2 /* Passthrough */ )
+    if ( !form.elements['newMonitor[VideoWriter]'] || form.elements['newMonitor[VideoWriter]'].value == 2 /* Passthrough */ )
       errors[errors.length] = "<?php echo translate('BadPassthrough') ?>";
   } else if ( form.elements['newMonitor[Type]'].value == 'Remote' ) {
     //if ( !form.elements['newMonitor[Host]'].value || !form.elements['newMonitor[Host]'].value.match( /^[0-9a-zA-Z_.:@-]+$/ ) )
@@ -100,44 +82,51 @@ function validateForm() {
       errors[errors.length] = "<?php echo translate('BadPath') ?>";
     } else if (form.elements['newMonitor[Path]'].value.match(/[\!\*'\(\)\$ ,#]/)) {
       warnings[warnings.length] = "<?php echo translate('BadPathNotEncoded') ?>";
-    } else {
-      try {
-        let url = new URL(form.elements['newMonitor[Path]'].value);
-        if (url.href != form.elements['newMonitor[Path]'].value) {
-          warnings[warnings.length] = "<?php echo translate('BadPathNotEncoded') ?>";
-        }
-      } catch(err) {
-        warnings[warnings.length] = err.message;
-      }
     }
+/*
+ * Alternate way of testing for bad urls
+    let url = new URL(form.elements['newMonitor[Path]'].value);
+    if (url.href != form.elements['newMonitor[Path]'].value) {
+      warnings[warnings.length] = "<?php echo translate('BadPathNotEncoded') ?>";
+    }
+*/
+
   } else if ( form.elements['newMonitor[Type]'].value == 'File' ) {
     if ( !form.elements['newMonitor[Path]'].value )
       errors[errors.length] = "<?php echo translate('BadPath') ?>";
     if ( form.elements['newMonitor[VideoWriter]'].value == 2 /* Passthrough */ )
       errors[errors.length] = "<?php echo translate('BadPassthrough') ?>";
   } else if ( form.elements['newMonitor[Type]'].value == 'WebSite' ) {
-    if ( form.elements['newMonitor[Function]'].value != 'Monitor' && form.elements['newMonitor[Function]'].value != 'None')
-      errors[errors.length] = "<?php echo translate('BadSourceType') ?>";
+    //if ( form.elements['newMonitor[Function]'].value != 'Monitor' && form.elements['newMonitor[Function]'].value != 'None')
+      //errors[errors.length] = "<?php echo translate('BadSourceType') ?>";
     if ( form.elements['newMonitor[Path]'].value.search(/^https?:\/\//i) )
       errors[errors.length] = "<?php echo translate('BadWebSitePath') ?>";
   }
+
   if ( form.elements['newMonitor[Type]'].value != 'WebSite' ) {
-    if ( !form.elements['newMonitor[Colours]'].value || (parseInt(form.elements['newMonitor[Colours]'].value) != 1 && parseInt(form.elements['newMonitor[Colours]'].value) != 3 && parseInt(form.elements['newMonitor[Colours]'].value) != 4 ) )
+    if (form.elements['newMonitor[VideoWriter]'].value == '1' /* Encode */) {
+      const parameters = form.elements['newMonitor[EncoderParameters]'].value.replace(/[^#a-zA-Z]/g, "");
+      if (parameters == '' || parameters == '#Linesbeginningwith#areacomment#Forchangingqualityusethecrfoption#isbestisworstquality#crf' ) {
+        warnings[warnings.length] = '<?php echo translate('BadEncoderParameters') ?>';
+      }
+    }
+
+    if ( form.elements['newMonitor[AnalysisFPSLimit]'].value && !(parseFloat(form.elements['newMonitor[AnalysisFPSLimit]'].value) >= 0 ) )
+      errors[errors.length] = "<?php echo translate('BadAnalysisFPS') ?>";
+    if ( form.elements['newMonitor[MaxFPS]'].value && !(parseFloat(form.elements['newMonitor[MaxFPS]'].value) >= 0 ) )
+      errors[errors.length] = "<?php echo translate('BadMaxFPS') ?>";
+    if ( form.elements['newMonitor[AlarmMaxFPS]'].value && !(parseFloat(form.elements['newMonitor[AlarmMaxFPS]'].value) >= 0 ) )
+      errors[errors.length] = "<?php echo translate('BadAlarmMaxFPS') ?>";
+    if ( !form.elements['newMonitor[RefBlendPerc]'].value || (parseInt(form.elements['newMonitor[RefBlendPerc]'].value) > 100 ) || (parseInt(form.elements['newMonitor[RefBlendPerc]'].value) < 0 ) )
+      errors[errors.length] = "<?php echo translate('BadRefBlendPerc') ?>";
+
+    const colours = form.elements['newMonitor[Colours]'];
+    if (!colours || !colours.value || (parseInt(colours.value) != 1 && parseInt(colours.value) != 3 && parseInt(colours.value) != 4))
       errors[errors.length] = "<?php echo translate('BadColours') ?>";
     if ( !form.elements['newMonitor[Width]'].value || !(parseInt(form.elements['newMonitor[Width]'].value) > 0 ) )
       errors[errors.length] = "<?php echo translate('BadWidth') ?>";
     if ( !form.elements['newMonitor[Height]'].value || !(parseInt(form.elements['newMonitor[Height]'].value) > 0 ) )
       errors[errors.length] = "<?php echo translate('BadHeight') ?>";
-  }
-
-  if (errors.length) {
-    $j('#source-tab').tab('show');
-    alert(errors.join("\n"));
-    return false;
-  }
-
-  if ( form.elements['newMonitor[Type]'].value != 'WebSite' ) {
-
     if ( !form.elements['newMonitor[LabelX]'].value || !(parseInt(form.elements['newMonitor[LabelX]'].value) >= 0 ) )
       errors[errors.length] = "<?php echo translate('BadLabelX') ?>";
     if ( !form.elements['newMonitor[LabelY]'].value || !(parseInt(form.elements['newMonitor[LabelY]'].value) >= 0 ) )
@@ -189,13 +178,7 @@ function validateForm() {
     return false;
   }
 
-  if (form.elements['newMonitor[VideoWriter]'].value == '1' /* Encode */) {
-    var parameters = form.elements['newMonitor[EncoderParameters]'].value.replace(/[^#a-zA-Z]/g, "");
-    if (parameters == '' || parameters == '#Linesbeginningwith#areacomment#Forchangingqualityusethecrfoption#isbestisworstquality#crf' ) {
-      warnings[warnings.length] = '<?php echo translate('BadEncoderParameters') ?>';
-    }
-  }
-  if ( (form.elements['newMonitor[Function]'].value != 'Monitor') && (form.elements['newMonitor[Function]'].value != 'None') ) {
+  if ( (form.elements['newMonitor[Recording]'].value != 'None') ) {
     if ( (form.elements['newMonitor[SaveJPEGs]'].value == '0') && (form.elements['newMonitor[VideoWriter]'].value == '0') ) {
       warnings[warnings.length] = "<?php echo translate('BadNoSaveJPEGsOrVideoWriter'); ?>";
     }
@@ -205,16 +188,25 @@ function validateForm() {
       return false;
     }
   }
-  form.elements['action'].value='save';
-  form.submit();
+
+  /* because of success here, we will submit the form. Before we do that,
+   * convert all password fields to hidden fields so that
+   * browsers don't offer to save them
+   */
+
+  for (let i=0; i < form.elements.length; i++) {
+    if (form.elements[i].type == 'password') {
+      form.elements[i].type = 'hidden';
+    }
+  }
   return true;
 }
 
 function updateMethods(element) {
-  var form = element.form;
+  const form = element.form;
 
-  var origMethod = form.elements['origMethod'];
-  var methodSelector = form.elements['newMonitor[Method]'];
+  const origMethod = form.elements['origMethod'];
+  const methodSelector = form.elements['newMonitor[Method]'];
   methodSelector.options.length = 0;
   switch ( element.value ) {
     case 'http' :
@@ -242,5 +234,12 @@ function updateMethods(element) {
       ?>
     break;
   }
+  if (element.value == 'rtsp') {
+    $j('#RTSPDescribe').show();
+  } else {
+    $j('#RTSPDescribe').hide();
+  }
   return true;
 }
+const monitors = <?php global $monitors; echo isset($monitors) ? json_encode($monitors) : '{}' ?>;
+const zones = <?php global $zones; echo isset($zones) ? json_encode($zones) : '{}' ?>;
