@@ -70,7 +70,7 @@ if ( !isset($newZone) ) {
 			'Units'	=>	'Pixels',
       'MonitorId' => $monitor->Id(),
       'NumCoords' => 4,
-      'Coords' => sprintf('%d,%d %d,%d %d,%d %d,%d', $minX, $minY, $maxX, $minY, $maxX, $maxY, $minX, $maxY),
+      'Coords' => sprintf('%d,%d %d,%d, %d,%d %d,%d', $minX, $minY, $maxX, $minY, $maxX, $maxY, $minX, $maxY),
       'Area' => $monitor->ViewWidth() * $monitor->ViewHeight(),
       'AlarmRGB' => 0xff0000,
       'CheckMethod' => 'Blobs',
@@ -109,16 +109,15 @@ $selfIntersecting = isSelfIntersecting($newZone['Points']);
 $focusWindow = true;
 # Have to do this here, because the .js.php references somethings figured out when generating the streamHTML
 $monitor->connKey();
-$StreamHTML = getStreamHTML($monitor, array('mode'=>'single'));
 
 # So I'm thinking now that 50% of screen real-estate with a minimum of 640px. 
 # scale should be floor(whatever that width is/actual width)
 # So we need javascript to figure out browser width, figure out scale and then activate the stream.  
 
 xhtmlHeaders(__FILE__, translate('Zone'));
+getBodyTopHTML();
+echo getNavBarHTML();
 ?>
-<body>
-  <?php echo getNavBarHTML() ?>
   <div id="page">
     <div class="w-100">
       <div class="float-left pl-3 pt-1">
@@ -141,12 +140,11 @@ xhtmlHeaders(__FILE__, translate('Zone'));
         <input type="hidden" name="newZone[Area]" value="<?php echo $newZone['Area'] ?>"/>
         <input type="hidden" name="newZone[AlarmRGB]"/>
 
-        <div id="definitionPanel">
-				  <div id="imagePanel">
-            <div class="Monitor">
-              <div id="monitor<?php echo $monitor->Id() ?>">
-            <div id="imageFrame<?php echo $monitor->Id()?>" class="imageFrame">
-              <?php echo $StreamHTML; ?>
+				  <div id="imageAndPoints">
+            <div id="imageFrame">
+              <?php echo 
+$monitor->getStreamHTML(array('mode'=>'single', 'zones'=>false, 'state'=>true));
+?>
               <svg id="zoneSVG" class="zones" viewBox="0 0 <?php echo $monitor->ViewWidth().' '.$monitor->ViewHeight() ?>">
 <?php
 if ( $zone['Id'] ) {
@@ -166,21 +164,62 @@ if ( count($other_zones) ) {
                 <polygon id="zonePoly" points="<?php echo $zone['AreaCoords'] ?>" class="Editing <?php echo $zone['Type'] ?>"/>
                 Sorry, your browser does not support inline SVG
               </svg>
-            </div><?php # imageFrame?>
-            <div id="monitorState">
-              <?php echo translate('State') ?>:&nbsp;<span id="stateValue<?php echo $monitor->Id() ?>"></span>&nbsp;-&nbsp;<span id="fpsValue<?php echo $monitor->Id() ?>"></span>&nbsp;fps
-            </div>
+            </div><?php # imageFrame
+?>
+
             <div id="StreamControlButtons">
-              <button type="button" id="pauseBtn" title="<?php echo translate('Pause') ?>">
+              <button type="button" id="analyseBtn" class="btn btn-primary" title="<?php echo translate('Showing Analysis') ?>">
+                <i class="material-icons md-18">assessment</i>
+              </button>
+              <button type="button" id="pauseBtn" class="btn btn-primary" title="<?php echo translate('Pause') ?>">
                 <i class="material-icons md-18">pause</i>
               </button>
-              <button type="button" id="playBtn" title="<?php echo translate('Play') ?>">
+              <button type="button" id="playBtn" class="btn btn-primary" title="<?php echo translate('Play') ?>">
                 <i class="material-icons md-18">play_arrow</i>
               </button>
             </div>
-            </div>
-            </div>
-          </div><!--imagePanel-->
+            <div id="zonePoints">
+              <table>
+                <tbody>
+                  <tr>
+  <?php
+  $pointCols = 2;
+  for ( $i = 0; $i < $pointCols; $i++ ) {
+  ?>
+                    <td>
+                      <table>
+                        <thead>
+                          <tr>
+                            <th><?php echo translate('Point') ?></th>
+                            <th><?php echo translate('X') ?></th>
+                            <th><?php echo translate('Y') ?></th>
+                            <th><?php echo translate('Action') ?></th>
+                          </tr>
+                        </thead>
+                        <tbody>
+                        </tbody>
+                      </table>
+                    </td>
+  <?php
+  # I think this for horizontal filler
+      if ( $i < ($pointCols-1) ) {
+  ?>
+                    <td>&nbsp;</td>
+  <?php
+      }
+  } # end foreach pointcol
+  ?>
+                  </tr>
+                </tbody>
+              </table>
+              <div class="buttons">
+                <button type="button" id="saveBtn" value="Save" <?php if (!canEdit('Monitors') || (false && $selfIntersecting)) { ?> disabled="disabled"<?php } ?>>
+                <?php echo translate('Save') ?>
+                </button>
+                <button type="button" id="cancelBtn" value="Cancel"><?php echo translate('Cancel') ?></button>
+              </div>
+            </div><!--end ZonePoints-->
+          </div><!--image & points-->
 
 					<div id="settingsPanel">
 						<table id="zoneSettings">
@@ -273,51 +312,16 @@ if ( count($other_zones) ) {
 							</tbody>
 						</table>
 					</div>
-
-					<div id="zonePoints">
-					  <table>
- 					 	  <tbody>
-					 		  <tr>
-<?php
-$pointCols = 2;
-for ( $i = 0; $i < $pointCols; $i++ ) {
-?>
-									<td>
-										<table>
-											<thead>
-												<tr>
-													<th><?php echo translate('Point') ?></th>
-													<th><?php echo translate('X') ?></th>
-													<th><?php echo translate('Y') ?></th>
-													<th><?php echo translate('Action') ?></th>
-												</tr>
-											</thead>
-											<tbody>
-											</tbody>
-										</table>
-									</td>
-<?php
-# I think this for horizontal filler
-    if ( $i < ($pointCols-1) ) {
-?>
-									<td>&nbsp;</td>
-<?php
-    }
-} # end foreach pointcol
-?>
-								</tr>
-							</tbody>
-						</table>
-					<div class="buttons">
-						<button type="button" id="saveBtn" value="Save" <?php if (!canEdit('Monitors') || (false && $selfIntersecting)) { ?> disabled="disabled"<?php } ?>>
-						<?php echo translate('Save') ?>
-						</button>
-						<button type="button" id="cancelBtn" value="Cancel"><?php echo translate('Cancel') ?></button>
-					</div>
-					</div><!--end ZonePoints-->
-        </div><!--definitionsPanel-->
       </form>
     </div><!--content-->
   </div><!--page-->
+<?php
+if ($monitor->JanusEnabled()) {
+?>
+  <script src="<?php echo cache_bust('js/adapter.min.js') ?>"></script>
+  <script src="/javascript/janus/janus.js"></script>
+<?php
+}
+?>
   <script src="<?php echo cache_bust('js/MonitorStream.js') ?>"></script>
 <?php xhtmlFooter() ?>
